#!/usr/bin/python3
# IBM CONFIDENTIAL
#
# OBJECT CODE ONLY SOURCE MATERIALS
#
# COPYRIGHT International Business Machines Corp. 2015
# All Rights Reserved
#
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.
'''
This program reads a file containing IO adapter information
and assigns adapters accordingly to 2 VIOS servers.
'''

from gettext import gettext as _
from pvmconfigdata import PVMConfigData as pcd
import ast
import getopt
import six
import sys


def assign_to_vios(iolist, rlist, io_redundant):
    '''
    Obtains drc_name from iolist then assign to VIOS1 first then
    VIOS2. If there is only one drc_name then VIOS1 gets it.
    :param iolist: List of I/O adapter to assign
    :param rlist: Dictionary with 2 keys 'VIOS1' and 'VIOS2'
    :param io_redundant: Boolean to indicate whether or not io redundancy
           is set. If True assign to 2 VIOS servers, else only assign to
           one VIOS server.
    '''
    tovios1 = True
    vios1list = rlist['VIOS1']
    vios2list = rlist['VIOS2']
    if io_redundant:
        for drc in iolist:
            if tovios1:
                vios1list.append(drc)
                tovios1 = False
            else:
                vios2list.append(drc)
                tovios1 = True
    else:
        for drc in iolist:
            vios1list.append(drc)


def assign_to_sdn(desc_list_, rlist_):
    '''
    Assigns the I/O adapters to SDN1.  Currently this method does not assign
    any adapter to SDN2.
    @param desc_list_: List of IO adapters with loc codes and descriptions
    @param rlist_: List of SDN LPARs
    '''
    # From the DRC name list for each of the type of adapter,
    # assign them to SDN1.
    sdn1list = rlist_['SDN1']
    if len(desc_list_) > 0:
        for k_ in desc_list_:
            iol_ = desc_list_[k_]
            for drc_ in iol_:
                sdn1list.append(drc_)
        rc_ = 0
    else:
        six.print_(_("Empty list of io adapters in file"), file=sys.stderr)
        rc_ = 1
    return rc_


def printusage():
    six.print_(_("Usage: %s -f infile -o outfile [-n] [-h | --help]") %
               sys.argv[0], file=sys.stderr)
    six.print_(_("     -f    : Input file containing io adapter information"),
               file=sys.stderr)
    six.print_(_("     -o    : Output file to write io assignment to"),
               file=sys.stderr)
    six.print_(_("     -n    : Flag to indicate that the script is invoked in "
                 "non greenfield install mode"))
    six.print_(_("-h, --help : Prints this help message"), file=sys.stderr)
    sys.exit(2)


if __name__ == '__main__':
    infile = ""
    ofile = ""
    non_greenfield_sdn = False
    rc = 0

    try:
        options, remainder = getopt.getopt(sys.argv[1:], 'f:o:nh',
                                           ['help'])
    except getopt.GetoptError as e:
        six.print_(str(e), file=sys.stderr)
        printusage()

    for opt, arg in options:
        if opt in ('-h', '--help'):
            printusage()
        elif opt in ('-f'):
            infile = arg
        elif opt in ('-o'):
            ofile = arg
        elif opt in ('-n'):
            non_greenfield_sdn = True

    if not infile:
        six.print_(_("Missing input file"), file=sys.stderr)
        printusage()
    if not ofile:
        six.print_(_("Missing output file"), file=sys.stderr)
        printusage()
    else:
        try:
            cfgparser = pcd(ofile, raw=True)
        except ValueError:
            six.print_(_("Unable to parse output file"), file=sys.stderr)
            printusage()
    # Now parse the input file and build 2 data structures
    # drc_list will be a dictionary with key the drc name
    # and value the description
    # desc_list is a dictionary with key the description
    # and value, a list of drc_name since we can have multiple
    # adapters of the same description type
    drc_list = {}
    desc_list = {}
    f = open(infile, 'r')
    for line in f:
        if line:
            (a, b) = line.split(":")
            d = b.strip()
            drc_list[a] = d
            val = []
            if d in desc_list:
                val = desc_list[d]
                val.append(a)
                desc_list[d] = val
            else:
                val.append(a)
                desc_list[d] = val

    # Need to determine if redundant io is desired.
    # If not assign all adapter to VIOS1
    redundant = cfgparser.get_attribute(pcd.PVM_CONFIG_SECTION,
                                        pcd.IOREDUNDANCY)
    io_red = True if redundant == '' else ast.literal_eval(str(redundant))

    if (non_greenfield_sdn):
        rlist = {'SDN1': [], 'SDN2': []}
        rc = assign_to_sdn(desc_list, rlist)
    else:
        rlist = {'VIOS1': [], 'VIOS2': []}
        rc = 0
        # From the DRC name list for each of the type of adapter,
        # assign them to VIOS1 then VIOS2 respectively.
        for k in desc_list:
            iol = desc_list[k]
            assign_to_vios(iol, rlist, io_red)
        if len(desc_list) <= 0:
            six.print_(_("Empty list of io adapters in file"), file=sys.stderr)
            rc = 1
    installmode = cfgparser.get_attribute(pcd.PVM_CONFIG_SECTION,
                                          pcd.INSTALLMODE)
    # Now rlist has 2 entries representing list of drc index belonging
    # to VIOS1 and VIOS2.
    # Go thru the list and build a table of entries for each VIOS,
    # having DRC index keys, with value DRC name and IO slot description
    if not rc and not non_greenfield_sdn:
        vios1info = {}
        iolist = rlist['VIOS1']
        for drc in iolist:
            val = drc_list[drc]
            vios1info[drc] = val
        vios2info = {}
        iolist = rlist['VIOS2']
        for drc in iolist:
            val = drc_list[drc]
            vios2info[drc] = val
        # In SDE mode, we need to set -
        # SDN_ENABLED = True
        # IO_REDUNDANT = False
        # Assign all physical I/O to NovaLink (SDN1)
        if installmode == pcd.SDE:
            # Set SDN_ENABLED to true in SDE mode
            cfgparser.set_attribute(pcd.SDN_CONFIG_SECTION,
                                    pcd.SDN_ENABLED, True)
            # Set IO_REDUNDANT to false in SDE mode
            cfgparser.set_attribute(pcd.PVM_CONFIG_SECTION,
                                    pcd.IOREDUNDANCY, False)
            # Assign all Physical I/O to NovaLink LPAR
            sdn1info = vios1info.copy()
            sdn1info.update(vios2info)
            cfgparser.set_attribute(pcd.SDN1_PHYS_IO_CONFIG_SECTION,
                                    pcd.INFO, sdn1info)
        else:
            cfgparser.set_attribute(pcd.VIOS1_PHYS_IO_CONFIG_SECTION,
                                    pcd.INFO, vios1info)
            cfgparser.set_attribute(pcd.VIOS2_PHYS_IO_CONFIG_SECTION,
                                    pcd.INFO, vios2info)
    elif non_greenfield_sdn:
        sdn1info = {}
        iolist = rlist['SDN1']
        for drc in iolist:
            val = drc_list[drc]
            sdn1info[drc] = val
        sdn2info = {}
        iolist = rlist['SDN2']
        for drc in iolist:
            val = drc_list[drc]
            sdn2info[drc] = val
        if installmode == pcd.SDN:
            cfgparser.set_attribute(pcd.SDN1_PHYS_IO_CONFIG_SECTION,
                                    pcd.INFO, sdn1info)
            cfgparser.set_attribute(pcd.SDN2_PHYS_IO_CONFIG_SECTION,
                                    pcd.INFO, sdn2info)
    sys.exit(rc)
