# IBM CONFIDENTIAL
#
# OBJECT CODE ONLY SOURCE MATERIALS
#
# COPYRIGHT International Business Machines Corp. 2016
# All Rights Reserved
#
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.

import re
from snack import GridForm
from snack import TextboxReflowed
from snack import CheckboxTree
from gettext import gettext as _
from wizpkg.wizardpanel import WizardPanel
from wizpkg.extendedwizardpanels import buildButtonBar
from diskselect.targetdiskinstall import StorageDevicesException
from diskselect.targetdiskinstall import ExistingPartitionException


class TargetDiskInstallWizardPanel(WizardPanel):
    '''
    This class extends the base WizardPanel and uses
    one or more Checkboxes with a description as widget in its gridform.
    Multiple selections are allowed.
             +-------------------------------+
             |     some title                |
             +-------------------------------+
             | Here are some instructions..  |
             |                               |
             | [ ] Some text                 |
             | [ ] Some other text           |
             | ...                           |
             |                               |
             | <back>   <next>   <cancel>    |
             +-------------------------------+
    '''
    def __init__(self, screen, name, paneltitle, diskDev, position,
                 panelaction):
        '''
        Init method
        :param screen: SnackScreen instance
        :param name: Name of the panel
        :param paneltitle: Panel title displayed to user
        :param diskDev: TargetDiskInstall object
               Contains list of disk, list of selected disks
               and padding to display neat columns
        :param position: Panel position in wizard
        :param panelaction: Function to called when user transition out
        '''

        msg = _("Select the disk(s) to be used for installation. Either 1 "
                "disk (no RAID) or 2 disks (RAID 1) can be selected. For "
                "multipath disks only one selection is allowed. All data "
                "on the disk(s) selected will be erased.")

        gf = GridForm(screen, paneltitle, 1, 3)
        tb = TextboxReflowed(WizardPanel.WIDTH, msg)
        self.targetdisk = diskDev
        height = len(self.targetdisk.diskList)
        scroll = 0
        # cap displayed interfaces to 8 at a time. display scrollbar
        if height > 8:
            height = 8
            scroll = 1
        self.cbt = CheckboxTree(height=height, scroll=scroll)
        # selected disk stored in configuration file has the dev
        # string in the name and can have more than one disk.
        # First convert string into a list in order to use
        # it later for deciding to set the selected flag
        seldisks = self.targetdisk.selectedDisks.split()

        # Iterate through disk entries
        for diskentry in self.targetdisk.diskList:
            # format disk info into neat columns
            # Padding values were calculated when data was initially parsed
            disktext = "{d[0]:{pname}} {d[1]:{psize}} {d[2]:{pdesc}}" \
                       " {d[3]} {d[4]}".format(d=diskentry,
                                               pname=self.targetdisk.padName,
                                               psize=self.targetdisk.padSize,
                                               pdesc=self.targetdisk.padDesc)
            # Truncate string to keep it on the screen
            # length 70 allows for checkbox and scrollbar to fill as well
            disktext = disktext[:70]
            # diskentry[0] = name
            # diskentry[1] = size
            # diskentry[2] = desc
            # diskentry[3] = vend
            # diskentry[4] = prod
            # add the /dev/ or /dev/mapper/ to the name in order to compare
            if "mpath" in diskentry[0]:
                diskname = "/dev/mapper/" + diskentry[0]
            else:
                diskname = "/dev/" + diskentry[0]
            selected = 0
            if diskname in seldisks:
                selected = 1

            # CheckboxTree: (text, item = None, selected = 0)
            self.cbt.append(disktext, diskname, selected)

        gf.add(tb, 0, 0, padding=(0, 0, 0, 1), anchorTop=0)
        gf.add(self.cbt, 0, 1,)
        bb = buildButtonBar(screen, position)
        gf.add(bb, 0, 2, padding=(0, 1, 0, 0), growx=1)
        self.buttonBar = bb
        super(TargetDiskInstallWizardPanel, self).__init__(
            screen=screen, name=name, gridform=gf, panelaction=panelaction)

    def getValues(self):
        '''
        Obtain values selected
        This will be the list of key values for all the checkboxes in the tree
        '''
        selectedDisks = self.cbt.getSelection()

        # Build list of selected disks, taking care of just returning
        # the disk name from the selection returned since we need to
        # append it to /dev/ or as is in the case of mpio device
        selectedDisksPath = ''
        for disk in selectedDisks:
            if "mpath" in disk:
                selectedDisksPath += disk + " "
            else:
                selectedDisksPath += ("/dev/%s " % re.sub('/dev/', '', disk))
        return selectedDisksPath

    def validate(self):
        '''
        This method validates user selection
        No more than two disks can be selected
        An mpath disk cannot be selected with another disk
        Additionally warns user if selected disk has
        existing partitions
        '''
        # use string instead of array
        selectedDisks = self.getValues()
        rc = 0
        try:
            self.targetdisk.validate(selectedDisks)
            self.targetdisk.checkPartitions(selectedDisks)
        except StorageDevicesException as e:
            self.showError(self.screen, str(e))
            rc = 1
        except ExistingPartitionException:
            rc = 2
        return rc
