#!/usr/bin/python3
# IBM CONFIDENTIAL
#
# OBJECT CODE ONLY SOURCE MATERIALS
#
# COPYRIGHT International Business Machines Corp. 2015
# All Rights Reserved
#
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.
from xml.etree import ElementTree
from gettext import gettext as _
import getopt
import re
import sys
import six
import shlex


def printusage():
    six.print_(_("Usage: %s [ -f file ] {[ -m file ] [ -s file]}"
                 " [ -h | --help ]")
               % sys.argv[0],
               file=sys.stderr)
    six.print_(_("     -f    : data file from lshw output"), file=sys.stderr)
    six.print_(_("     -m    : data file from multipath output"),
               file=sys.stderr)
    six.print_(_("     -s    : data file from ls /sys/block/dm-*/slaves/"),
               file=sys.stderr)
    six.print_(_("-h , --help: Prints this help message"), file=sys.stderr)
    sys.exit(2)


def getattribute(node, attribute):
    '''
    Get attribute value from element in the tree
    :param node: child element of the root node from parsing
           the xml file
    :param attribute: name of the attribute to obtain
    :returns: An empty string if no attribute is found, else
              the attribute value
    '''
    try:
        return node.find(attribute).text
    except AttributeError:
        return ""


def getmpathslaves(slave_file):
    '''
    Parse ls /sys/block/dm-*/slaves/ output
    :param slave_file: datafile
    :returns: a dictionary{dm-#:slaves_disks}
    '''
    try:
        with open(slave_file, "r") as sfile:
            data = sfile.readlines()
    except IOError:
        six.print_(_("Unable to open %s") % slave_file)
        sys.exit(1)

    dm_slaves_dict = {}
    slaves = ""
    for lsout in data:
        # /sys/block/dm-2/slaves/sdc
        ln = lsout.strip()
        # dm-2
        device = ln.split('/')[3]
        # sdc
        disk = ln.split('/')[5]
        # |sda|sdb|
        slaves = "%s%s|" % (slaves, disk)
        # dm-X entries will be grouped
        # write over older entry each time
        dm_slaves_dict[device] = slaves

    return dm_slaves_dict


try:
    options, remainder = getopt.getopt(sys.argv[1:], 'f:hm:s:', ['help'])
except getopt.GetoptError as e:
    print(str(e))
    printusage()

mpath = False
for opt, arg in options:
    if opt in ('-h', '--help'):
        printusage()
    elif opt in ['-f']:
        datafile = arg
    elif opt in ['-m']:
        mpath = True
        mpathfile = arg
    elif opt in ['-s']:
        slavefile = arg

# Scan a file that contains the xml output from lshw -c storage
# and prints information about storage device using | as a field
# separator
if not mpath:
    try:
        with open(datafile, "r") as f:
            tree = ElementTree.parse(f)
            root = tree.getroot()
    except IOError:
        six.print_(_("Unable to open %s") % datafile)
        sys.exit(1)

    pattern = "^disk*"
    diskpatt = re.compile(pattern)

    for child in root.iter('node'):
        # Parse and filter out anything that is not
        # a disk. CDROM will show up as removable
        # disk, therefore 'cdrom' id will be ignored
        if diskpatt.match(child.attrib['id']):
            desc = getattribute(child, 'description')
            vendor = getattribute(child, 'vendor')
            product = getattribute(child, 'product')
            name = getattribute(child, 'logicalname')
            size = getattribute(child, 'size')
            # size is in bytes
            disksize = int(size) / (1024 * 1024) if size != "" else 0
            # Prints name without the /dev/ along with
            # description, vendor, product and size
            # sdX|SCSI Disk|IBM|2145|61440(MB)
            print("%s|%s|%s|%s|%d(MB)" %
                  (name[5:], desc, vendor, product, disksize))
else:
    try:
        with open(mpathfile, "r") as mfile:
            content = mfile.readlines()
    except IOError:
        six.print_(_("Unable to open %s") % mpathfile)
        sys.exit(1)

    namepatt = re.compile("^mpath*")
    sizepatt = re.compile("^size=*")
    # This file contains the mpio device name, minor number, size
    # The device minor number is than correlated with the
    # slave dictionary to determine slave disks
    slave_dict = getmpathslaves(slavefile)
    for line in content:
        # find start of mpath device
        if namepatt.match(line):
            (mpath_name, wwid, device_minor, product) = shlex.split(line)
        # next line will have size
        if sizepatt.match(line):
            (size, ft, hw, wp) = shlex.split(line)
            # Grab size in Gigabytes
            size = size.split("=")[1]
            # reformat size symbol
            mpath_size = "%s(%sB)" % (size[:-1], size[-1:])
            # Prints name without /dev/mapper/ along with
            # slave disks and size
            # mpathX|sdg|sdc|sdf|594(GB)
            print("%s|%s%s" % (mpath_name, slave_dict[device_minor],
                               mpath_size))

sys.exit(0)
