# IBM CONFIDENTIAL
#
# OBJECT CODE ONLY SOURCE MATERIALS
#
# COPYRIGHT International Business Machines Corp. 2015
# All Rights Reserved
#
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.

import ast
import crypt
import os
from gettext import gettext as _
from network.networkpanel import NetworkConfigWizardPanel
from network.network import Network
from pvmconfigdata import PVMConfigData as pcd
from pvmtime.time import PVMTime
from summary import SummaryPanel
from snack import CompactButton
from snack import ButtonBar
from confirminstall import ConfirmInstallPanel
from user.userpanel import UserPasswordWizardPanel
from user.user import User
from vios.ioredundant import IORedundantWizardPanel
from vios.ioredundant import IORedundantSamplePanel
from sdn.doSdn import DoSDNWizardPanel
from wizpkg.wizardpanel import WizardPanel
from ioadapter.iopanel import IOWizardPanel
from ioadapter.physadapter import PhysicalAdapter
from procmem.procmempanel import ProcMemoryWizardPanel
from procmem.virtprocinfo import VirtProcInfo
from procmem.meminfo import MemInfo
from wizpkg.progressbar import CommandProgressWindow
from wizpkg.progressbar import ExecWithProgress
from wizpkg.extendedwizardpanels import AcknowledgementWizardPanel
from networkbridge.networkbridge import run_network_bridge
from network.networkdevicepanel import NetworkDeviceWizardPanel
from network.ethernetdevice import EthernetDevice
from network.ethernetdevice import NoEthernetDevicesFoundException
from diskselect.targetdiskinstallpanel import TargetDiskInstallWizardPanel
from diskselect.targetdiskinstall import TargetDisk
from diskselect.targetdiskinstall import StorageDevicesException

'''
PVM Install wizard
   - Prompts for User and Password information
   - Prompts for Network information
'''

PADMIN = 'padmin'


def _exec_with_progress(command, argv, screen, stdin=0, stdout=1, stderr=2,
                        pBar=None, root='/'):
    argv = list(argv)
    if isinstance(stdin, str):
        if os.access(stdin, os.R_OK):
            stdin = open(stdin)
        else:
            stdin = 0

    if isinstance(stdout, str):
        stdout = open(stdout, "w+")
    else:
        stdout = 1
    if isinstance(stderr, str):
        stderr = open(stderr, "w+")
    else:
        stderr = 2

    # Create a pipe to facilitate capturing output of command

    p = os.pipe()
    childpid = os.fork()
    if not childpid:
        os.close(p[0])
        os.dup2(p[1], 1)
        os.dup2(stderr.fileno(), 2)
        os.dup2(stdin, 0)
        os.close(stdin)
        os.close(p[1])
        stderr.close()

        os.execvp(command, [command] + argv)
        os._exit(1)

    os.close(p[1])
    while 1:
        try:
            # Since the window is not too wide, we should limit to reading in
            # at most 50 characters
            s = os.read(p[0], 50)
        except OSError as e:
            (num, val) = e.args
            if (num != 4):
                raise IOError(str(e))

        # Update window with text and progress bar value
        if pBar:
            pBar.setText(str(s, encoding="utf-8"))
            pBar.update()

        if len(s) < 1:
            break
    try:
        (pid, status) = os.waitpid(childpid, 0)
    except OSError as e:
        WizardPanel.showError(screen, _("An exception has occurred from "
                                        "running %s" % command))
    pBar and pBar.pop()

    if status is None:
        return 0
    # One should never get here
    if os.WIFEXITED(status):
        return os.WEXITSTATUS(status)
    return 1


def _setup_vios(screen, ioredundant):
    rc = 0
    title = _("Setup VIOS")
    text = _("Accept License, deploy rules and restart VIOS")
    w = CommandProgressWindow(screen, title, text, 100, 25, update=True)

    rc = ExecWithProgress.runcmd("/opt/ibm/pvm-install/bin/vioscfg",
                                 ['-o', 'l', '-i', '2'], screen,
                                 stdout="/tmp/exec.out",
                                 stderr="/tmp/exec.err", pBar=w)
    if rc != 0:
        WizardPanel.showError(screen, _("An error has occurred. Unable "
                                        "to setup VIOS 1."))
        return rc
    if ioredundant:
        w = CommandProgressWindow(screen, title, text, 100, 25, update=True)

        rc = ExecWithProgress.runcmd("/opt/ibm/pvm-install/bin/vioscfg"
                                     ['-o', 'l', '-i', '3'], screen,
                                     stdout="/tmp/exec.out",
                                     stderr="/tmp/exec.err", pBar=w)
        if rc != 0:
            WizardPanel.showError(screen, _("An error has occurred. Unable "
                                            "to setup VIOS 2."))
            return rc
    return rc


def _createvios(screen):
    title = _("Create VIOS")
    text = _("Creating VIOS Server(s)...")
    w = CommandProgressWindow(screen, title, text, 100, 5, update=True)

    rc = ExecWithProgress.runcmd("/opt/ibm/pvm-install/bin/vioscfg",
                                 ['-o', 'c'], screen, stdout="/tmp/exec.out",
                                 stderr="/tmp/exec.err", pBar=w)
    return rc


def _createswitchcfg(screen):
    title = _("Create Network Install")
    text = _("Setting up private network for VIOS server install...")
    w = CommandProgressWindow(screen, title, text, 100, 25, update=True)

    rc = ExecWithProgress.runcmd("/opt/ibm/pvm-install/bin/switchnetworkcfg",
                                 ['-o', 'r'], screen, stdout="/tmp/exec.out",
                                 stderr="/tmp/exec.err", pBar=w)
    return rc


def _monitor_install_progress(screen):
    title = _("Install VIOS")
    text = _("Reference codes during installation")
    w = CommandProgressWindow(screen, title, text, 300, 1, update=True)
    rc = ExecWithProgress.runcmd("/opt/ibm/pvm-install/bin/vioscfg",
                                 ['-o', 'm'], screen, stdout="/tmp/exec.out",
                                 stderr="/tmp/exec.err", pBar=w)
    return rc


def _poweronvios(screen, lparid):
    title = _("Power on partition %s" % lparid)
    text = _("Powering on VIOS server...")
    w = CommandProgressWindow(screen, title, text, 100, 25, update=True)

    rc = ExecWithProgress.runcmd("/opt/ibm/pvm-install/bin/vioscfg",
                                 ['-o', 'o', '-i', str(lparid)], screen,
                                 stdout="/tmp/exec.out",
                                 stderr="/tmp/exec.err", pBar=w)
    return rc


def _restart_vios(screen, lparid):
    title = _("Reboot partition %s" % lparid)
    text = _("Rebooting VIOS server...")
    w = CommandProgressWindow(screen, title, text, 100, 25, update=True)

    rc = ExecWithProgress.runcmd("/opt/ibm/pvm-install/bin/vioscfg",
                                 ['-o', 's', '-i', str(lparid)], screen,
                                 stdout="/tmp/exec.out",
                                 stderr="/tmp/exec.err", pBar=w)
    return rc


def _build_network_panel(screen, cfp, network, pvmtime):

    if not network.dhcp_protocol:
        network.dhcp_protocol = cfp.get_attribute(
            pcd.NETWORK_SETTINGS_SECTION, pcd.DHCPIP)
    if not network.ipaddress:
        network.ipaddress = cfp.get_attribute(
            pcd.NETWORK_SETTINGS_SECTION, pcd.IPADDR)
    if not network.gateway:
        network.gateway = cfp.get_attribute(pcd.NETWORK_SETTINGS_SECTION,
                                            pcd.GATEWAY)
    if not network.networkmask:
        network.networkmask = cfp.get_attribute(
            pcd.NETWORK_SETTINGS_SECTION, pcd.NETMASK)
    if not network.dns1:
        network.dns1 = cfp.get_attribute(pcd.NETWORK_SETTINGS_SECTION,
                                         pcd.DNS1)
    if not network.dns2:
        network.dns2 = cfp.get_attribute(pcd.NETWORK_SETTINGS_SECTION,
                                         pcd.DNS2)
    if not network.dns3:
        network.dns3 = cfp.get_attribute(pcd.NETWORK_SETTINGS_SECTION,
                                         pcd.DNS3)
    if not network.hostname:
        network.hostname = cfp.get_attribute(
            pcd.NETWORK_SETTINGS_SECTION, pcd.HOSTNAME)
    if not network.domainname:
        network.domainname = cfp.get_attribute(
            pcd.NETWORK_SETTINGS_SECTION, pcd.DOMAIN)

    # NTP information collected in network panel
    if not pvmtime.ntpserver:
        pvmtime.ntpserver = cfp.get_attribute(
            pcd.GENERAL_SETTINGS_SECTION, pcd.NTPSERVER)
    if not pvmtime.timezone:
        pvmtime.timezone = cfp.get_attribute(
            pcd.GENERAL_SETTINGS_SECTION, pcd.TIMEZONE)
    # If NTP enabled is NOTSET then relies on what is in the config file
    if pvmtime.ntpenabled == 'NOTSET':
        pvmtime.ntpenabled = cfp.get_attribute(
            pcd.GENERAL_SETTINGS_SECTION, pcd.NTPENABLED)

    return NetworkConfigWizardPanel(screen, 'NETWORK_CONFIG',
                                    _("Network Configuration"),
                                    _("Choose whether to dynamically obtain "
                                      "the ip address by selecting DHCP, "
                                      "or provide static IP address "
                                      "information"), pvmtime, network, 1,
                                    panelaction=_network_config_action)


def _build_vios_network_panel(screen, cfp, vios_network, cfg_section):

    if not vios_network.dhcp_protocol:
        vios_network.dhcp_protocol = cfp.get_attribute(cfg_section, pcd.DHCPIP)
    if not vios_network.ipaddress:
        vios_network.ipaddress = cfp.get_attribute(cfg_section, pcd.IPADDR)
    if not vios_network.gateway:
        vios_network.gateway = cfp.get_attribute(cfg_section, pcd.GATEWAY)
    if not vios_network.networkmask:
        vios_network.networkmask = cfp.get_attribute(cfg_section, pcd.NETMASK)
    if not vios_network.dns1:
        vios_network.dns1 = cfp.get_attribute(cfg_section, pcd.DNS1)
    if not vios_network.dns2:
        vios_network.dns2 = cfp.get_attribute(cfg_section, pcd.DNS2)
    if not vios_network.dns3:
        vios_network.dns3 = cfp.get_attribute(cfg_section, pcd.DNS3)
    if not vios_network.hostname:
        vios_network.hostname = cfp.get_attribute(cfg_section, pcd.HOSTNAME)
    if not vios_network.domainname:
        vios_network.domainname = cfp.get_attribute(cfg_section, pcd.DOMAIN)

    if cfg_section == pcd.VIOS1_NETW_SETTINGS_SECTION:
        title = _("Network Configuration (VIOS 1)")
    else:
        title = _("Network Configuration (VIOS 2)")

    return NetworkConfigWizardPanel(screen, 'NETWORK_CONFIG', title,
                                    _("Choose whether to dynamically obtain "
                                      "the ip address by selecting DHCP, "
                                      "or provide static IP address "
                                      "information"), None,
                                    vios_network, 1,
                                    panelaction=_network_config_action)


def _assign_io_to_vios1(cfp):
    '''
    Assign all io adapters to the first VIOS
    This method is called when user choose to not have i/o redundancy
    '''
    vios1_pio = cfp.get_attribute(pcd.VIOS1_PHYS_IO_CONFIG_SECTION, pcd.INFO)
    vios2_pio = cfp.get_attribute(pcd.VIOS2_PHYS_IO_CONFIG_SECTION, pcd.INFO)
    vios1_ioinfo = {}
    vios2_ioinfo = {}
    tempcopy = {}
    if vios1_pio:
        vios1_ioinfo = ast.literal_eval(str(vios1_pio))
    if vios2_pio:
        vios2_ioinfo = ast.literal_eval(str(vios2_pio))
    # merge the 2 dictionary into one and use it for VIOS1
    # then clear VIOS2
    tempcopy = vios1_ioinfo.copy()
    tempcopy.update(vios2_ioinfo)
    cfp.set_attribute(pcd.VIOS1_PHYS_IO_CONFIG_SECTION,
                      pcd.INFO, tempcopy)
    cfp.set_attribute(pcd.VIOS2_PHYS_IO_CONFIG_SECTION,
                      pcd.INFO, {})


def _build_phys_adapter_info(cfp, vios1_info, vios2_info):

    MAX_ENTRY = IOWizardPanel.MAX_ENTRY
    vios1_pio = cfp.get_attribute(pcd.VIOS1_PHYS_IO_CONFIG_SECTION,
                                  pcd.INFO)
    vios2_pio = cfp.get_attribute(pcd.VIOS2_PHYS_IO_CONFIG_SECTION,
                                  pcd.INFO)
    ioinfo = {}
    vios1_drc_list = []
    vios2_drc_list = []
    if vios1_pio:
        vios1_ioinfo = ast.literal_eval(str(vios1_pio))
        for k in vios1_ioinfo:
            vios1_drc_list.append(k)
        ioinfo = vios1_ioinfo.copy()
    if vios2_pio:
        vios2_ioinfo = ast.literal_eval(str(vios2_pio))
        for k in vios2_ioinfo:
            vios2_drc_list.append(k)
        ioinfo.update(vios2_ioinfo)
    max_rows = len(ioinfo)
    start = 0
    end = MAX_ENTRY if max_rows > MAX_ENTRY else max_rows
    return PhysicalAdapter(ioinfo, vios1_drc_list,
                           vios2_drc_list, start, end)


def _build_user_password_panel(screen, cfp, pvmuser, viosuser,
                               show_vios):

    username = pvmuser.username
    if not username:
        username = cfp.get_attribute(pcd.CREDENTIALS_SECTION,
                                     pcd.USERNAME)
    passwd1 = pvmuser.passwd1
    passwd2 = pvmuser.passwd2
    passwd3 = viosuser.passwd1
    passwd4 = viosuser.passwd2

    if show_vios:
        msg = _("Enter the administrator user name and "
                "password for the NovaLink partition and "
                "both Virtual I/O Servers.")
    else:
        msg = _("Enter the administrator user name and "
                "password for the NovaLink partition.")
    return UserPasswordWizardPanel(screen, 'USERPASSWORD',
                                   _("User and Password"),
                                   msg, username, passwd1, passwd2, passwd3,
                                   passwd4, show_vios, 1,
                                   panelaction=_user_setting_action)


def _process_commands(screen, show_vios, cfgfile, ioredundant):
    # Not used but want to leave it here in case we can find a better
    # way to show progress yet able to interrupt progress window
    rc = 0
    rc = os.system("/opt/ibm/pvm-install/bin/pvmconfigdata.py -r -f " +
                   cfgfile + " -o d -s VIOSNIMServerConfig > "
                   "/opt/ibm/pvm-install/data/novalinknetw.data")
    if rc != 0:
        WizardPanel.showError(screen, _("An error has occurred. "
                                        "Unable to initialize private network "
                                        "configuration data."))
        return rc

    if show_vios:
        rc = _createvios(screen)
        if rc != 0:
            WizardPanel.showError(screen, _("An error has occurred. "
                                            "Unable to create "
                                            "VIOS Server(s)."))
            return rc
        rc = _createswitchcfg(screen)
        if rc != 0:
            WizardPanel.showError(screen, _("An error has occurred. "
                                            "Unable to create "
                                            "network to install VIOS "
                                            "Server(s)."))
            return rc
        rc = _poweronvios(screen, 2)
        if rc != 0:
            WizardPanel.showError(screen, _("An error has occurred. "
                                            "Unable to power on "
                                            "VIOS 1."))
            return rc
        if ioredundant:
            rc = _poweronvios(screen, 3)
            if rc != 0:
                WizardPanel.showError(screen, _("An error has occurred. "
                                                "Unable to power on "
                                                "VIOS 2."))
                return rc

        cfp = pcd(cfgfile, raw=True)
        attrval = cfp.get_attribute(pcd.VIOSNIMSERVER_CONFIG_SECTION,
                                    pcd.VIOSINSTALLPROMPT)
        viosprompt = ast.literal_eval(str(attrval))
        if viosprompt:
            access_shell(screen)

        rc = _monitor_install_progress(screen)
        if rc != 0:
            WizardPanel.showError(screen, _("An error has occurred "
                                            "during monitor of VIOS install."))
            return rc

        rc = _setup_vios(screen, ioredundant)
        if rc != 0:
            WizardPanel.showError(screen, _("An error has occurred "
                                            "during VIOS setup."))
            return rc

    # Now preseed information for novalink install
    rc = os.system("/opt/ibm/pvm-install/bin/updatedebconf "+cfgfile)
    # If there is an error then we should back up to the last
    # panel and let user decides.
    if rc != 0:
        WizardPanel.showError(screen, _("An error has occurred. "
                                        "Unable to update "
                                        "configuration database "
                                        "needed to start the "
                                        "installation."))
        return rc
    return rc


def _network_config_action(cfp, nwobj, timeobj, cfgsection):
    '''
    Action to run after validating network entry
    from NetworkConfigPanel
    '''
    ipaddr = nwobj.ipaddress
    gateway = nwobj.gateway
    netmask = nwobj.networkmask
    dhcpproto = nwobj.dhcp_protocol
    domainname = nwobj.domainname
    hostname = nwobj.hostname
    dns1 = nwobj.dns1
    dns2 = nwobj.dns2
    dns3 = nwobj.dns3

    cfp.set_attribute(cfgsection, pcd.HOSTNAME, hostname)
    if domainname:
        cfp.set_attribute(cfgsection, pcd.DOMAIN, domainname)
    cfp.set_attribute(cfgsection, pcd.DHCPIP, dhcpproto)
    if ipaddr:
        cfp.set_attribute(cfgsection, pcd.IPADDR, ipaddr)
    if netmask:
        cfp.set_attribute(cfgsection, pcd.NETMASK, netmask)
    if gateway:
        cfp.set_attribute(cfgsection, pcd.GATEWAY, gateway)
    if dns1:
        cfp.set_attribute(cfgsection, pcd.DNS1, dns1)
    if dns2:
        cfp.set_attribute(cfgsection, pcd.DNS2, dns2)
    if dns3:
        cfp.set_attribute(cfgsection, pcd.DNS3, dns3)

    if timeobj:
        ntpserver = timeobj.ntpserver
        if ntpserver:
            cfp.set_attribute(pcd.GENERAL_SETTINGS_SECTION,
                              pcd.NTPSERVER, ntpserver)
            cfp.set_attribute(pcd.GENERAL_SETTINGS_SECTION,
                              pcd.NTPENABLED, True)
        else:
            cfp.set_attribute(pcd.GENERAL_SETTINGS_SECTION,
                              pcd.NTPENABLED, False)


def _network_device_action(cfp, ethdev):
    cfp.set_attribute(pcd.PVM_CONFIG_SECTION,
                      pcd.TARGET_NETWORK_DEVICE, ethdev.selected_port)


def _target_disk_action(cfp, targetDisk):
    # targetDisks.selectDisks is a string. if 2 disks, they are space seperated
    cfp.set_attribute(pcd.PVM_CONFIG_SECTION,
                      pcd.TARGET_DISK_INSTALL, targetDisk.selectedDisks)


def _procmeminfo_action(cfp, pName, procinfo, meminfo):
    '''
    Action to run after validating virtual proc and
    memory selected by user
    '''
    if pName == 'NovaLinkProcMemWizardPanel':
        section = pcd.PVM_CONFIG_SECTION
    elif pName == 'VIOSProcMemWizardPanel':
        section = pcd.VIOS_CONFIG_SECTION

    cfp.set_attribute(section, pcd.VIRTUALPROCDESIRED,
                      procinfo.desiredvirtproc)
    cfp.set_attribute(section, pcd.DESIREDMEMORY,
                      meminfo.desiredmemory)


def _user_setting_action(cfp, pvmuser, viosuser):
    '''
    Action to run after validating user entry from
    UserPasswordPanel
    '''
    cfp.set_attribute(pcd.CREDENTIALS_SECTION, pcd.USERNAME, pvmuser.username)
    # Need to crypt the password before writing
    # to the file
    encryptedpw = crypt.crypt(pvmuser.passwd1, crypt.METHOD_SHA512)
    cfp.set_attribute(pcd.CREDENTIALS_SECTION, pcd.PASSWORD, encryptedpw)
    # For now use CRYPT if need to enhance to SHA512 then we will
    # need a way to see what Hash encryption user want
    vioshashpw = crypt.crypt(viosuser.passwd1, crypt.METHOD_CRYPT)
    cfp.set_attribute(pcd.VIOS_CREDENTIALS_SECTION, pcd.PASSWORD, vioshashpw)


def _process_user_panel(screen, cfp, pvmuser, viosuser, show_vios):
    '''
    Process user/password panel. Return one of the following values:
        WizardPanel.ERROR
        WizardPanel.EXIT
        WizardPanel.BACK
        WizardPanel.NEXT
    '''
    panel = _build_user_password_panel(screen, cfp, pvmuser,
                                       viosuser, show_vios)
    panel.run()
    nextpanel = panel.next()
    if nextpanel == WizardPanel.BACK:
        return WizardPanel.BACK
    elif nextpanel == WizardPanel.CANCEL:
        return WizardPanel.EXIT
    else:
        pEntryList = panel.getValues()
        pvmuser.username = pEntryList[0]
        pvmuser.passwd1 = pEntryList[1]
        pvmuser.passwd2 = pEntryList[2]
        viosuser.username = PADMIN
        viosuser.passwd1 = pEntryList[3]
        viosuser.passwd2 = pEntryList[4]
        rc = panel.validate()
        if rc != 0:
            return WizardPanel.ERROR
        else:
            panel.action(cfp, pvmuser, viosuser)
            return WizardPanel.NEXT


def _process_network_panel(screen, cfp, network, pvmtime):
    '''
    Process data on network panel. Return one of the following values:
        WizardPanel.ERROR
        WizardPanel.EXIT
        WizardPanel.BACK
        WizardPanel.NEXT
    '''
    panel = _build_network_panel(screen, cfp, network, pvmtime)
    panel.run()
    nextpanel = panel.next()
    if nextpanel == WizardPanel.BACK:
        return WizardPanel.BACK
    elif nextpanel == WizardPanel.CANCEL:
        return WizardPanel.EXIT
    else:
        pEntryList = panel.getValues()
        # Persist values so they can be shown in case of error
        # or going back
        network.dhcp_protocol = pEntryList[0]
        network.hostname = pEntryList[1]
        network.domainname = pEntryList[2]
        network.ipaddress = pEntryList[3]
        network.gateway = pEntryList[4]
        network.networkmask = pEntryList[5]
        network.dns1 = pEntryList[6]
        network.dns2 = pEntryList[7]
        network.dns3 = pEntryList[8]
        pvmtime.ntpserver = pEntryList[9]
        rc = panel.validate()
        if rc != 0:
            return WizardPanel.ERROR
        else:
            # Run panel action with:
            # dhcpip (4 or 6), hostname, domainname, ipaddr, netmask,
            # gateway, dns1, dns2, dns3
            panel.action(cfp, network, pvmtime,
                         pcd.NETWORK_SETTINGS_SECTION)
            return WizardPanel.NEXT


def _process_vios_network_panel(screen, cfp, vios_network, cfg_section):
    '''
    Process data on vios network panel. Return one of the following values:
        WizardPanel.ERROR
        WizardPanel.EXIT
        WizardPanel.BACK
        WizardPanel.NEXT
    '''
    panel = _build_vios_network_panel(screen, cfp, vios_network,
                                      cfg_section)
    panel.run()
    nextpanel = panel.next()
    if nextpanel == WizardPanel.BACK:
        return WizardPanel.BACK
    elif nextpanel == WizardPanel.CANCEL:
        return WizardPanel.EXIT
    else:
        pEntryList = panel.getValues()
        # Persist values so they can be shown in case of error
        # or going back
        vios_network.dhcp_protocol = pEntryList[0]
        vios_network.hostname = pEntryList[1]
        vios_network.domainname = pEntryList[2]
        vios_network.ipaddress = pEntryList[3]
        vios_network.gateway = pEntryList[4]
        vios_network.networkmask = pEntryList[5]
        vios_network.dns1 = pEntryList[6]
        vios_network.dns2 = pEntryList[7]
        vios_network.dns3 = pEntryList[8]
        rc = panel.validate()
        if rc != 0:
            return WizardPanel.ERROR
        else:
            # Run panel action with:
            # dhcpip (4 or 6), hostname, domainname, ipaddr, netmask,
            # gateway, dns1, dns2, dns3
            panel.action(cfp, vios_network, None, cfg_section)
            return WizardPanel.NEXT


def _process_procmem_panel(screen, cfp, pName, procinfo, meminfo):
    '''
    Asks user for memory and virtual processor values
    Returns one of:
        WizardPanel.ERROR
        WizardPanel.EXIT
        WizardPanel.BACK
        WizardPanel.NEXT
    '''
    panel_msgs = []
    if pName == 'NovaLinkProcMemWizardPanel':
        panel_name = 'NovaLink_PROC_MEM_INFO'
        panel_title = _("NovaLink Processors and Memory")
        panel_msgs.append(_("Specify virtual processor and memory allocation "
                            "for the NovaLink LPAR. By default, entitled "
                            "processors are 50% of virtual processor "
                            "allocation. Suggested values:"))
        panel_msgs.append(_("  Small:  2 virtual processors, 4 GB  memory\n"
                            "  Medium: 4 virtual processors, 8 GB  memory\n"
                            "  Large:  8 virtual processors, 16 GB memory\n"))
    elif pName == 'VIOSProcMemWizardPanel':
        panel_name = 'VIOS_PROC_MEM_INFO'
        panel_title = _("VIOS Processors and Memory")
        panel_msgs.append(_("Specify virtual processor and memory allocation "
                            "for one or both Virtual I/O Servers. By default, "
                            "entitled processors are 50% of virtual processor "
                            "allocation. Suggested values:"))
        panel_msgs.append(_("   Small:  2 virtual processors, 4 GB  memory\n"
                            "   Medium: 4 virtual processors, 8 GB  memory\n"
                            "   Large:  8 virtual processors, 16 GB memory\n"))

    panel_action = _procmeminfo_action
    panel = ProcMemoryWizardPanel(screen, panel_name, panel_title, panel_msgs,
                                  procinfo, meminfo,
                                  1, panelaction=panel_action)
    panel.run()
    nextpanel = panel.next()
    pEntryList = panel.getValues()
    procinfo.desiredvirtproc = pEntryList[0]
    meminfo.desiredmemory = pEntryList[1]

    if nextpanel == WizardPanel.BACK:
        return WizardPanel.BACK
    elif nextpanel == WizardPanel.CANCEL:
        return WizardPanel.EXIT
    else:
        rc = panel.validate()
        if rc != 0:
            return WizardPanel.ERROR
        else:
            panel.action(cfp, pName, procinfo, meminfo)
            return WizardPanel.NEXT


def _process_network_device_panel(screen, cfp, ethdev):
    '''
    Asks user for network interface selection
    Returns one of:
        WizardPanel.ERROR
        WizardPanel.EXIT
        WizardPanel.BACK
        WizardPanel.NEXT
    '''
    try:
        ethdev.findethernetdevices()
    except NoEthernetDevicesFoundException as e:
        WizardPanel.showError(screen, str(e))
        return WizardPanel.BACK

    user_port = cfp.get_attribute(pcd.PVM_CONFIG_SECTION,
                                  pcd.TARGET_NETWORK_DEVICE)
    # Override if user specified device
    if user_port != "":
        ethdev.selected_port = user_port

    panel = NetworkDeviceWizardPanel(screen, 'NETWORK_DEV',
                                     _("Network Interface"), ethdev, 1,
                                     panelaction=_network_device_action)

    panel.run()
    nextpanel = panel.next()
    pEntryList = panel.getValues()
    ethdev.selected_port = pEntryList[0]
    if nextpanel == WizardPanel.BACK:
        return WizardPanel.BACK
    elif nextpanel == WizardPanel.CANCEL:
        return WizardPanel.EXIT
    else:
        panel.action(cfp, ethdev)
        return WizardPanel.NEXT


def _process_physicalio_panel(screen, cfp):
    '''
    Asks user to allocate physical io to the vios
    This should be called ONLY if redundant io is desired
    Returns one of:
        WizardPanel.ERROR
        WizardPanel.EXIT
        WizardPanel.BACK
        WizardPanel.NEXT
    '''
    MAX_ENTRY = IOWizardPanel.MAX_ENTRY

    # Config section names
    cfg_sections = {"VIOS1": pcd.VIOS1_PHYS_IO_CONFIG_SECTION,
                    "VIOS2": pcd.VIOS2_PHYS_IO_CONFIG_SECTION,
                    "SDN1": pcd.SDN1_PHYS_IO_CONFIG_SECTION,
                    "SDN2": pcd.SDN2_PHYS_IO_CONFIG_SECTION}

    # A dictionary of dictionaries to hold all the pre-partitioned
    # adapter dictionaries
    io_assign_dict = {}

    # A dictionary of all adapters, with each is a DRC and
    # Description pair
    adapterdict = {}

    # Read configuration file and construct the two dictionaries:
    # io_assign_dict, adapterdict
    for owner, cfg_section in cfg_sections.items():

        cfg_pio_per_owner = cfp.get_attribute(cfg_section, pcd.INFO)
        pio_per_owner = {}
        if cfg_pio_per_owner:
            pio_per_owner = ast.literal_eval(str(cfg_pio_per_owner))

        io_assign_dict.update({owner: pio_per_owner})
        adapterdict.update(pio_per_owner)

    # Calculate boundary of the sliding window over all the adapters
    start = 0
    max_rows = len(adapterdict)
    end = MAX_ENTRY if max_rows > MAX_ENTRY else max_rows

    # Determines the owners
    owner_names = []
    owner_names = _calc_header_labels(cfp)

    # "physadapter" is the object holding the two dictionaries
    physadapter = PhysicalAdapter(owner_names, adapterdict,
                                  io_assign_dict, start, end)

    # Execute the physical adapter selection wizard
    while True:

        # Data flow of the physical adapter selection wizard:
        # Input:  dictionaries of adapters in the physadapter object
        # Output: a dictionary of dictionaries representing the selected
        #         partitioning of the adapters among the active owners.
        panel = IOWizardPanel(screen, 'PHYSICAL_ADAPTERS',
                              _("Physical Adapters"),
                              _("Allocate I/O adapters to exactly one "
                                "owner."),
                              physadapter)
        panel.run()
        nextpanel = panel.next()
        out_io_assign_dict = panel.getValues()

        # The new adapter assignment is written back to physadapter
        physadapter.update_assignment(out_io_assign_dict.copy())

        # Process the button bar selection to determine whether to
        # keep looping or done with this wizard
        tobreak = _process_physicalio_buttonbar(nextpanel, physadapter)

        if tobreak:
            break

    # Write the adapter partitioning info, carried in physadapter,
    # back into the configuration file.
    if nextpanel == WizardPanel.BACK or nextpanel == WizardPanel.NEXT:
        _update_io_info(cfp, physadapter)

    elif nextpanel == WizardPanel.CANCEL:
        nextpanel = WizardPanel.EXIT

    return nextpanel


def _process_physicalio_buttonbar(nextpanel, physadapter):
    '''
    Processes the button bar selection of the physical adapter selection
    wizard.
    :param nextpanel: a selected buttone bar action
    :param physadapter: holds adapter range displayed in the screen
              On return, this holds adapter range to be displayed in the
              next screen
    :return: boolean value indicating whether to break out of the wizard loop
    '''
    tobreak = False
    MAX_ENTRY = IOWizardPanel.MAX_ENTRY
    max_rows = len(physadapter.adapterdict)

    if nextpanel == WizardPanel.PREV:
        physadapter.start = physadapter.start - MAX_ENTRY
        physadapter.end = physadapter.start + MAX_ENTRY

    elif nextpanel == WizardPanel.MORE:
        physadapter.start = physadapter.start + MAX_ENTRY
        cond = (physadapter.end + MAX_ENTRY > max_rows)
        val = (physadapter.end + MAX_ENTRY)
        physadapter.end = max_rows if cond else val
    else:
        tobreak = True

    return tobreak


def _update_io_info(cfp, physadapter):

    cfg_sections = {"VIOS1": pcd.VIOS1_PHYS_IO_CONFIG_SECTION,
                    "VIOS2": pcd.VIOS2_PHYS_IO_CONFIG_SECTION,
                    "SDN1": pcd.SDN1_PHYS_IO_CONFIG_SECTION,
                    "SDN2": pcd.SDN2_PHYS_IO_CONFIG_SECTION}

    for owner, cfg_section in cfg_sections.items():
        if (cfp.config.has_section(cfg_section)):
            if owner not in physadapter.io_assign_dict:
                physadapter.io_assign_dict[owner] = {}
            cfp.set_attribute(cfg_section, pcd.INFO,
                              physadapter.io_assign_dict[owner])


def _calc_header_labels(cfp):
    '''
    Determines the names of the column headers in the IO adapter selection
    panel.

    :param cfp: Config Parser object
    :return: List of labels for the column headers in the io adapter selection
              panel.
    '''

    # Array to be returned
    owner_names = []

    # Retrieve the 4 owner enablement configuration variables
    sdn_enabled, sdn_redun_enabled, ioredundant, vios_enabled = \
        parse_sdn_attributes(cfp)

    # Mask vector
    owner_mask = {"VIOS1": vios_enabled,
                  "VIOS2": ioredundant,
                  "SDN1": sdn_enabled,
                  "SDN2": sdn_redun_enabled}

    # The 4 owners are rendered in order
    owner_list = ["VIOS1", "VIOS2", "SDN1", "SDN2"]

    # Keep the enabled owners
    for owner in owner_list:
        if owner_mask[owner]:
            owner_names.append(owner)

    return owner_names


def _process_ioredundant_panel(screen, cfp, sdn_enabled=False):
    '''
    Asks user whether or not i/o redundancy (Dual VIOS) is desired.
    Returns one of:
        WizardPanel.ERROR
        WizardPanel.EXIT
        WizardPanel.BACK
        WizardPanel.NEXT
    '''
    iopbb = ButtonBar(screen, ((_("Back"), WizardPanel.BACK),
                               (_("Next"), WizardPanel.NEXT),
                               (_("View example"), WizardPanel.EDIT),
                               (_("Cancel"), WizardPanel.CANCEL)),
                      compact=1)
    redundant = cfp.get_attribute(pcd.PVM_CONFIG_SECTION,
                                  pcd.IOREDUNDANCY)
    # Panel description message if SDN is enabled
    panel_description = _("Setting up I/O redundancy will result in load"
                          " balancing capabilities for multipath I/O (MPIO)")
    if (not sdn_enabled):
        panel_description = _("Setting up I/O redundancy will result in load"
                              " balancing capabilities for multipath I/O "
                              "(MPIO) and multiple Ethernet Bridges for"
                              " bridging to the physical network.")

    while True:
        panel = IORedundantWizardPanel(screen, 'IOREDUNDANT_PANEL',
                                       _("I/O Redundancy"),
                                       panel_description,
                                       redundant, iopbb, 1,
                                       panelaction=None)
        panel.run()
        nextpanel = panel.next()
        if nextpanel == WizardPanel.EDIT:
            _show_ioredundant_sample(screen)
        elif nextpanel == WizardPanel.BACK:
            return WizardPanel.BACK
        elif nextpanel == WizardPanel.CANCEL:
            return WizardPanel.EXIT
        else:
            val = panel.getValues()
            ioredundant = (False if val == IORedundantWizardPanel.NOREDUNDANT
                           else True)
            cfp.set_attribute(pcd.PVM_CONFIG_SECTION,
                              pcd.IOREDUNDANCY, ioredundant)
            if not ioredundant:
                _assign_io_to_vios1(cfp)
            return WizardPanel.NEXT


def _process_dosdn_panel(screen, cfp):
    '''
    Asks user whether or not to install SDN
    Returns one of:
        WizardPanel.ERROR
        WizardPanel.EXIT
        WizardPanel.BACK
        WizardPanel.NEXT
    '''
    sdnbb = ButtonBar(screen, ((_("Back"), WizardPanel.BACK),
                               (_("Next"), WizardPanel.NEXT),
                               (_("Cancel"), WizardPanel.CANCEL)),
                      compact=1)
    doSdn = cfp.get_attribute(pcd.SDN_CONFIG_SECTION,
                              pcd.SDN_ENABLED)
    panel = DoSDNWizardPanel(screen, 'DOSDN_PANEL',
                             _("Software Defined Networking (SDN)"),
                             _("Enable Software Defined Networking (SDN)"),
                             doSdn, sdnbb,
                             panelaction=None)
    panel.run()
    nextpanel = panel.next()
    if nextpanel == WizardPanel.BACK:
        return WizardPanel.BACK
    elif nextpanel == WizardPanel.CANCEL:
        return WizardPanel.EXIT
    else:
        val = panel.getValues()
        doSdn = (False if val == DoSDNWizardPanel.NOSDN else True)
        cfp.set_attribute(pcd.SDN_CONFIG_SECTION,
                          pcd.SDN_ENABLED, doSdn)
        return WizardPanel.NEXT


def _process_target_install_panel(screen, cfp, targetDisk):
    '''
    Process target disk install selection panel
    Return one of the following values:
        WizardPanel.ERROR
        WizardPanel.EXIT
        WizardPanel.BACK
        WizardPanel.NEXT
    '''
    if targetDisk.error:
        WizardPanel.showError(screen, targetDisk.error)
        return WizardPanel.BACK

    grub_selected = cfp.get_attribute(pcd.PVM_CONFIG_SECTION,
                                      pcd.TARGET_DISK_INSTALL)
    # set to grub selection if exists
    # overwrites default selection
    if grub_selected:
        targetDisk.selectedDisks = grub_selected

    paneltitle = _("Target Disk Install Selection")
    panel = TargetDiskInstallWizardPanel(screen, "TARGET_INSTALL_PANEL",
                                         paneltitle, targetDisk,
                                         1, panelaction=_target_disk_action)
    panel.run()
    nextpanel = panel.next()
    if nextpanel == WizardPanel.BACK:
        return WizardPanel.BACK
    elif nextpanel == WizardPanel.CANCEL:
        return WizardPanel.EXIT

    # rc values:
    # 0 - no error
    # 1 - invalid selection
    # 2 - existing partition
    rc = panel.validate()
    if rc == 1:
        return WizardPanel.ERROR
    # Confirm with user that this is the correct disk.
    elif rc == 2 and show_partition_warning(screen, targetDisk.warningMsg):
        return WizardPanel.ERROR
    targetDisk.selectedDisks = panel.getValues()
    panel.action(cfp, targetDisk)
    return WizardPanel.NEXT


def _process_summary_panel(screen, cfgfile, show_vios):
    '''
    Process data on network panel. Return one of the following values:
        WizardPanel.ERROR
        WizardPanel.EXIT
        WizardPanel.BACK
        WizardPanel.NEXT
    '''
    summarybb = ButtonBar(screen,
                          ((_("Back"), WizardPanel.BACK),
                           (_("Finish"), WizardPanel.FINISH),
                           (_("Edit Settings"), WizardPanel.EDIT),
                           (_("Cancel"), WizardPanel.CANCEL)),
                          compact=1)
    while True:
        try:
            cfp = pcd(cfgfile, raw=True)
        except ValueError:
            WizardPanel.showError(screen, _("An error has occurred. "
                                            "Unable to parse configuration "
                                            "file."))
        panel = SummaryPanel(screen, 'SUMMARY_PANEL',
                             _("Summary"),
                             _("Review the information below, then press "
                               "'Finish' to start the installation.\n"
                               "If you wish to make "
                               "additional changes, select 'Edit settings' "
                               "to edit the installer's configuration file.\n"
                               "NOTE: This is an advanced option, use extra "
                               "care when making changes."), cfp,
                             summarybb, show_vios, -1, panelaction=None)
        panel.run()

        nextpanel = panel.next()
        if nextpanel == WizardPanel.EDIT:
            run_edit(screen, cfgfile)
        elif nextpanel == WizardPanel.CANCEL:
            return WizardPanel.EXIT
        else:
            return nextpanel


def run_edit(screen, cfgfile):
    screen.suspend()
    os.system("/usr/bin/clear;/opt/ibm/pvm-install/bin/editpvmconfig "+cfgfile)
    screen.resume()


def access_shell(screen):
    screen.suspend()
    msg = _("Use mkvterm --id 2 or 3 to access VIOS"
            "\(type 'exit' to return to wizard\)")
    os.system("/usr/bin/clear;"
              " echo "+msg+";ENV=/opt/ibm/pvm-install/bin/envsetup /bin/sh")
    screen.resume()


def _show_ioredundant_sample(screen):

    bb = CompactButton(_("Back"))
    IORedundantSamplePanel(screen, 'REDUNDANT_CHART',
                           _("Sample Redundant Storage and Network"),
                           "/opt/ibm/pvm-install/data/vios_diagram.txt", bb)


def run_pvm_install_wizard(screen, cfgfile, showall, greenfield, show_vios):
    '''
    This method populates data of all the wizard panels
    and control the running of the wizard.
    First define default data for all the panels
    '''
    nextpanel = WizardPanel.NEXT
    try:
        cfp = pcd(cfgfile, raw=True)
    except ValueError:
        WizardPanel.showError(screen, _("An error has occurred. "
                                        "Unable to parse configuration file."))
    # The following objects represents the 'in memory' values of what
    # is entered on each panel. This allows us to be able to go back
    # and preserve the values that had been previously entered.
    # One special case is that these values will be refresh once you reached
    # the last panel (SummaryPanel) because that offers and option to directly
    # edit and modify the installer configuration file being passed in to
    # the wizard initally. Because the file can be modified, the various
    # instances need to be updated to reflect the values from the file.
    # see method update_panel_values
    pvmuser = User('', '', '')
    viosuser = User('padmin', '', '')
    network = Network('', '', '', '', '', '', '', '', '')
    pvmtime = PVMTime('NOTSET', '', '')
    vios1_network = Network('', '', '', '', '', '', '', '', '')
    vios2_network = Network('', '', '', '', '', '', '', '', '')
    ethdev = EthernetDevice()
    ioredundant = True
    targetDisk = TargetDisk()

    # check installmode
    installmode = cfp.get_attribute(pcd.PVM_CONFIG_SECTION,
                                    pcd.INSTALLMODE)

    sde_enabled = ""
    if installmode == pcd.SDE:
        sde_enabled = installmode

    # configuration variables for SDN related installation steps
    sdn_section = pcd.SDN_CONFIG_SECTION
    sdn_select_enabled = (installmode == pcd.SDN)
    sdn_enabled = \
        cfp.get_attribute_notstr(sdn_section, pcd.SDN_ENABLED)

    sdn_redun_select_enabled = \
        cfp.get_attribute_notstr(sdn_section, pcd.SDN_REDUN_SEL_ENABLED)

    lmbsize = cfp.get_attribute(pcd.SYSCONFIG_SECTION, pcd.LMBSIZE)

    # sdn_redun_enabled = \
    #     cfp.get_attribute_notstr(sdn_section, pcd.SDN_REDUN_ENABLED)

    # Populate target disk data
    try:
        targetDisk.parseFile()
    except StorageDevicesException as e:
        targetDisk.error = str(e)

    # PanelList contains the list of panels in the desired order of execution.
    # List must be terminated with None
    if greenfield:
        if sde_enabled:
            # NovaLinkProcMemWizardPanel only shown during custom
            PanelList = ['UserPasswordWizardPanel',
                         'NetworkConfigWizardPanel',
                         'NovaLinkProcMemWizardPanel',
                         'NetworkBridgePanel',
                         'TargetDiskInstallWizardPanel',
                         'SDN1NetworkPanel',
                         'SDN2NetworkPanel',
                         'SummaryWizardPanel',
                         'None']
        else:
            PanelList = ['UserPasswordWizardPanel',
                         'NetworkConfigWizardPanel',
                         'DoSDNWizardPanel',
                         'NovaLinkProcMemWizardPanel',
                         'IORedundantWizardPanel',
                         'VIOSProcMemWizardPanel',
                         'IOWizardPanel',
                         'NetworkBridgePanel',
                         'VIOS1NetworkPanel',
                         'VIOS2NetworkPanel',
                         'SDN1NetworkPanel',
                         'SDN2NetworkPanel',
                         'SummaryWizardPanel',
                         'None']

    else:
        if sde_enabled:
            PanelList = ['UserPasswordWizardPanel',
                         'NetworkConfigWizardPanel',
                         'SummaryWizardPanel',
                         'None']
        else:
            PanelList = ['UserPasswordWizardPanel',
                         'NetworkConfigWizardPanel',
                         'DoSDNWizardPanel',
                         'NetworkBridgePanel',
                         'SummaryWizardPanel',
                         'None']

    listIndex = 0
    while True:
        pName = PanelList[listIndex]
        # User and password for PVM and VIOS
        if pName == 'UserPasswordWizardPanel':
            nextpanel = _process_user_panel(screen, cfp, pvmuser,
                                            viosuser, show_vios)
        # PVM Network Interface selection
        if pName == 'NetworkDeviceWizardPanel':
            nextpanel = _process_network_device_panel(screen, cfp, ethdev)

        # PVM Network information
        if pName == 'NetworkConfigWizardPanel':
            nextpanel = _process_network_panel(screen, cfp, network, pvmtime)

        # Ask if user wants SDN installed
        if pName == 'DoSDNWizardPanel' and sdn_select_enabled:
            nextpanel = _process_dosdn_panel(screen, cfp)
            sdn_enabled = \
                cfp.get_attribute_notstr(sdn_section, pcd.SDN_ENABLED)

        # Ask for NovaLink LPAR configuration if user wants SDN installed
        if ((pName == 'NovaLinkProcMemWizardPanel' and showall and sdn_enabled)
                or (pName == 'NovaLinkProcMemWizardPanel' and showall and
                    sde_enabled) or (pName == 'VIOSProcMemWizardPanel' and
                                     showall)):

            # Fill out procmeminfo for the NovaLink LPAR by getting
            # information from the config file.
            # If file does not have desiredvirtproc or desiredmem values for
            # Novalink or the given values are too low, then 2 virtual
            # processors and 4GB memory will be used
            # If file does not have value for VIOS1 then 2 virtual processors
            # and 4GB memory will be used
            if (pName == 'NovaLinkProcMemWizardPanel'):
                section = pcd.PVM_CONFIG_SECTION
            elif (pName == 'VIOSProcMemWizardPanel'):
                section = pcd.VIOS_CONFIG_SECTION

            desiredvp = cfp.get_attribute(section, pcd.VIRTUALPROCDESIRED)
            minvp = cfp.get_attribute(section, pcd.VIRTUALPROCMIN)
            maxvp = cfp.get_attribute(section, pcd.VIRTUALPROCMAX)
            procinfo = VirtProcInfo(minvp, maxvp,
                                    '2' if desiredvp == '' else desiredvp)
            desiredmem = cfp.get_attribute(section, pcd.DESIREDMEMORY)
            if (desiredmem == '' or ((pName == 'NovaLinkProcMemWizardPanel')
                                     and ast.literal_eval(desiredmem) < 4)):
                desiredmem = '4'
            minmem = cfp.get_attribute(section, pcd.MINMEMORY)
            maxmem = cfp.get_attribute(section, pcd.MAXMEMORY)
            meminfo = MemInfo(minmem, maxmem, desiredmem, lmbsize)
            nextpanel = _process_procmem_panel(screen, cfp,
                                               pName, procinfo, meminfo)

        # Ask if user wants SDN redundancy. This  functionality is not yet
        # implemented.
        if (pName == 'SDNRedundantWizardPanel' and showall and
                sdn_redun_select_enabled):
            listIndex += 1
            continue

        # Ask if user wants i/o redundancy
        if pName == 'IORedundantWizardPanel':
            nextpanel = _process_ioredundant_panel(screen, cfp,
                                                   sdn_enabled)
            ioredundant = \
                cfp.get_attribute_notstr(pcd.PVM_CONFIG_SECTION,
                                         pcd.IOREDUNDANCY)

        if pName == 'IOWizardPanel':
            show_io_panel = sdn_enabled or (ioredundant and showall)
            if show_io_panel:
                nextpanel = _process_physicalio_panel(screen, cfp)

        if pName == 'VIOS1NetworkPanel':
            cfg_section = pcd.VIOS1_NETW_SETTINGS_SECTION
            nextpanel = _process_vios_network_panel(screen, cfp,
                                                    vios1_network, cfg_section)

        if pName == 'VIOS2NetworkPanel' and ioredundant:
            cfg_section = pcd.VIOS2_NETW_SETTINGS_SECTION
            nextpanel = _process_vios_network_panel(screen, cfp,
                                                    vios2_network, cfg_section)

        if pName == 'NetworkBridgePanel' and (
                greenfield or sdn_enabled or sde_enabled):
            nextpanel = run_network_bridge(screen, cfp, greenfield)

        if pName == 'TargetDiskInstallWizardPanel':
            nextpanel = _process_target_install_panel(screen, cfp,
                                                      targetDisk)

        # For the sumary panel, we will use the actual configuration file
        # since we allow user to edit
        if pName == 'SummaryWizardPanel':
            nextpanel = _process_summary_panel(screen, cfgfile, show_vios)
            # Refresh cfp so we reflect any changes
            # made to cfgfile should we need to go back
            try:
                cfp = pcd(cfgfile, raw=True)
                update_panel_values(cfp, pvmuser, network, pvmtime)
            except ValueError:
                WizardPanel.showError(screen, _("An error has occurred. "
                                                "Unable to parse "
                                                "configuration file."))

        # Indication of last panel
        if pName == 'None':
            break
        # Determine what to do next
        if nextpanel == WizardPanel.BACK:
            # First panel case
            if listIndex == 0:
                return WizardPanel.RUNNING
            else:
                listIndex = listIndex - 1
                continue
        if nextpanel == WizardPanel.NEXT:
            listIndex = listIndex + 1
            continue
        if nextpanel in (WizardPanel.CANCEL, WizardPanel.EXIT):
            show_cancel(screen)
            continue
        if nextpanel == WizardPanel.FINISH:
            rc = show_install_confirm(screen, cfp, greenfield)
            if rc == WizardPanel.FINISH:
                return rc
            continue

    return WizardPanel.FINISH


def update_panel_values(cfgparser, pvmuser, network, pvmtime):

    pvmuser.username = cfgparser.get_attribute(pcd.CREDENTIALS_SECTION,
                                               pcd.USERNAME)
    section = pcd.NETWORK_SETTINGS_SECTION
    network.dhcp_protocol = \
        cfgparser.get_attribute(section, pcd.DHCPIP)
    network.ipaddress = cfgparser.get_attribute(section, pcd.IPADDR)
    network.gateway = cfgparser.get_attribute(section, pcd.GATEWAY)
    network.networkmask = cfgparser.get_attribute(section, pcd.NETMASK)
    network.dns1 = cfgparser.get_attribute(section, pcd.DNS1)
    network.dns2 = cfgparser.get_attribute(section, pcd.DNS2)
    network.dns3 = cfgparser.get_attribute(section, pcd.DNS3)
    network.hostname = cfgparser.get_attribute(section, pcd.HOSTNAME)
    network.domainname = cfgparser.get_attribute(section, pcd.DOMAIN)
    pvmtime.ntpserver = cfgparser.get_attribute(pcd.GENERAL_SETTINGS_SECTION,
                                                pcd.NTPSERVER)
    pvmtime.timezone = cfgparser.get_attribute(pcd.GENERAL_SETTINGS_SECTION,
                                               pcd.TIMEZONE)
    pvmtime.ntpenabled = cfgparser.get_attribute(pcd.GENERAL_SETTINGS_SECTION,
                                                 pcd.NTPENABLED)


def show_cancel(screen):
    '''
    Displays the panel to confirm reboot of system
    '''
    panelName = _("Cancel Installation")
    panelMessage = _("Cancelling the wizard will result in the "
                     "partition shutting down.\n"
                     "Do you want to continue?")
    cancelPanel = AcknowledgementWizardPanel(screen, panelName,
                                             panelMessage,
                                             panelaction=None)
    cancelPanel.run()
    nextpanel = cancelPanel.next()
    if nextpanel == WizardPanel.NEXT:
        # By suspending the screen and run shutdown here we avoid leaving
        # the terminal screen in a bad state
        screen.suspend()
        os.system("shutdown")
    else:
        return WizardPanel.NEXT


def _bld_confirm_panel(grnfield, sde_sdn_mode, screen, osinfo, cfp):
    if sde_sdn_mode:
        panel_message = _("The following will be installed on the NovaLink "
                          "partition:\n\n" + osinfo +
                          "\nDo you want to continue?")
        finish_panel = AcknowledgementWizardPanel(
            screen, _("Confirm Installation"), panel_message,
            panelaction=None)
    else:
        finish_panel = ConfirmInstallPanel(screen, osinfo, cfp, grnfield)
    return finish_panel


def show_install_confirm(screen, cfgparser, grnfield):
    '''
    Displays a confirmation panel prior to start the install
    '''
    with open("/pvm-root/novalink-os.info", "r") as osinfofile:
        osinfo = osinfofile.read()

    installmode = cfgparser.get_attribute(pcd.PVM_CONFIG_SECTION,
                                          pcd.INSTALLMODE)
    sde_sdn_mode = (installmode == pcd.SDE or installmode == pcd.SDN)
    finish_panel = _bld_confirm_panel(grnfield, sde_sdn_mode,
                                      screen, osinfo, cfgparser)
    finish_panel.run()
    nextpanel = finish_panel.next()
    if nextpanel == WizardPanel.NEXT or nextpanel == WizardPanel.FINISH:
        sel_val = finish_panel.getValues()
        if sel_val == ConfirmInstallPanel.INSTALL_RHEL:
            cfgparser.set_attribute(pcd.PVM_CONFIG_SECTION,
                                    pcd.INSTALL_OTHER_OS, 'True')
            cfgparser.set_attribute(pcd.PVM_CONFIG_SECTION,
                                    pcd.INSTALLATION_SOURCE,
                                    finish_panel.inst_src_entry.value())
            cfgparser.set_attribute(pcd.PVM_CONFIG_SECTION, pcd.KS_FILE,
                                    finish_panel.ks_entry.value())
            cfgparser.set_attribute(pcd.PVM_CONFIG_SECTION, pcd.OTHER_OS_NAME,
                                    'RHEL')
        elif sel_val == ConfirmInstallPanel.INSTALL_OTHER:
            cfgparser.set_attribute(pcd.PVM_CONFIG_SECTION,
                                    pcd.INSTALL_OTHER_OS, 'True')
            cfgparser.set_attribute(pcd.PVM_CONFIG_SECTION, pcd.OTHER_OS_NAME,
                                    'Other')
        return WizardPanel.FINISH
    else:
        return WizardPanel.NEXT


def show_partition_warning(screen, panelmsg):
    '''
    Displays a warning to user, when partitions exist on selected disk
    '''
    continue_install = True
    panelName = _("Existing Partitions")
    partitionPanel = AcknowledgementWizardPanel(screen, panelName, panelmsg,
                                                panelaction=None)
    partitionPanel.run()
    nextpanel = partitionPanel.next()
    if nextpanel == WizardPanel.NEXT:
        continue_install = False
    return continue_install


def parse_sdn_attributes(cfp):
    '''
    Reads the config file to determine the enabled/disabled status of
    SDN and VIOS for the purpose of allocating IO adapters.
    :param cfp: Config Parser object
    :return: List of boolean values representing enabled/disabled state of SDN,
            VIOS, and their redundancy.
    '''
    sdn_sec = pcd.SDN_CONFIG_SECTION
    sdn_enabled = cfp.get_attribute_notstr(sdn_sec, pcd.SDN_ENABLED)
    sdn_redun_enabled = cfp.get_attribute_notstr(sdn_sec,
                                                 pcd.SDN_REDUN_ENABLED)
    ioredundant = cfp.get_attribute_notstr(pcd.PVM_CONFIG_SECTION,
                                           pcd.IOREDUNDANCY)
    installmode = cfp.get_attribute(pcd.PVM_CONFIG_SECTION,
                                    pcd.INSTALLMODE)
    vios_enabled = (installmode == pcd.SDN)

    # Some basic logic verification
    if sdn_redun_enabled:
        sdn_enabled = True
    if ioredundant or (sdn_enabled and installmode != pcd.SDE):
        vios_enabled = True
    # Older config files do not have the InstallMode, so handle that special
    # case here:  if the InstallMode option itself does not exist, then assume
    #  that the operator is using a old config file and expects VIOS to be
    #  installed.
    if (not cfp.config.has_option(pcd.PVM_CONFIG_SECTION,
                                  pcd.INSTALLMODE)):
        vios_enabled = True

    return sdn_enabled, sdn_redun_enabled, ioredundant, vios_enabled
