# IBM CONFIDENTIAL
#
# OBJECT CODE ONLY SOURCE MATERIALS
#
# COPYRIGHT International Business Machines Corp. 2015
# All Rights Reserved
#
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.

import re
from gettext import gettext as _
from snack import ButtonBar
from snack import Grid
from snack import GridForm
from snack import Label
from snack import SingleRadioButton
from snack import TextboxReflowed
from wizpkg.wizardpanel import WizardPanel


def natural_order(s):
    '''
    Returns a list of naturally sorted items
    '''
    return [int(x) if x.isdigit() else x for x in re.findall(r'\D+|\d+', s)]


class IOWizardPanel(WizardPanel):

    '''
    This class extends the base WizardPanel and displays IO resources,
    adapters, for user to assign to active owners of IO resources.

      ----------------------------------------------------------
      |     title                                              |
      |---------------------------------------------------------
      |        Some instructions                               |
      |                                                        |
      | VIOS1 VIOS2 SDN1  SDN2  Location code   Description    |
      |  ( )   ( )   ( )   ( )  U79AB.xxxx      xxxxxx         |
      | .....                                                  |
      |                                                        |
      |  1 - 8 of 15                                           |
      |                                                        |
      | <view previous>  <view more>  <back>  <next>  <cancel> |
      |________________________________________________________|

    There is no scrolling capability in this complex widget
    therefore we can only display limited number of io
    per panel.
    '''

    MAX_ENTRY = 7

    def __init__(self, screen, name, paneltitle, msg, ioinfo):

        # Parameters worth explaining:
        # param: ioinfo is an object containing needed information
        #        about physical adapter.

        maxrows = ioinfo.end - ioinfo.start

        gf = GridForm(screen, paneltitle, 1, 5)
        tb = TextboxReflowed(WizardPanel.WIDTH, msg)

        # Line #1: table header

        # Draw single table header line for all adapter owners, e.g.
        # . SDN
        # . VIOS
        # . SDN, VIOS
        # . SDN, VIOS1, VIOS2
        # . SDN1, SDN2, VIOS
        # . SDN1, SDN2, VIOS1, VIOS2
        owner_names = ioinfo.owner_names.copy()
        header_labels = owner_names.copy()
        header_labels.append(_("Location Code"))
        header_labels.append(_("Description"))

        num_owners = len(owner_names)
        num_labels = len(header_labels)

        # Definitions of width of fields:
        # . The width of the wizard window is fixed as 72
        # . The width of the location code is fixed as 28
        # . The width of each radio button is fixed as 6
        # . The the description uses whatever left from the total width
        _BOTTOM_DASH_LEN = 72
        _BUTTON_DASH_LEN = 6
        _SLOT_DASH_LEN = 28
        _DESC_DASH_LEN = (_BOTTOM_DASH_LEN - _SLOT_DASH_LEN -
                          num_owners * _BUTTON_DASH_LEN)
        _MAX_TEXT_DESC = _DESC_DASH_LEN

        g2 = Grid(num_labels, maxrows + 2)
        g2.setField

        for (label_idx, label) in enumerate(header_labels):
            g2.setField(Label(label), label_idx, 0,
                        padding=(0, 0, 0, 0), anchorLeft=1)

        # Line #2: table header border (separator of header and table body)
        header_borders = ["-" * _BUTTON_DASH_LEN] * num_owners
        header_borders.append("-" * _SLOT_DASH_LEN)
        header_borders.append("-" * _DESC_DASH_LEN)

        for (label_idx, border) in enumerate(header_borders):
            g2.setField(Label(border), label_idx, 1,
                        padding=(0, 0, 0, 0), anchorLeft=1)

        # Initialize some counters:
        # rowindex is used to obtain information from adapterdict
        # displayrow is used to keep track of current row in grid
        #    and starts at 2 because we added header and dash line
        # index is also a counter to adapterdict

        displayrow = 2
        adapterdict = ioinfo.adapterdict

        self.rbdicts = {}
        for owner_name in owner_names:
            self.rbdicts[owner_name] = {}

        io_assign_dict = ioinfo.io_assign_dict

        # Draw rows of physical adapters and selection/radio buttons
        for i in range(ioinfo.start, ioinfo.end):

            # TODO: check if need put a SORT here
            k, v = sorted(adapterdict.items())[i]

            # Create button objects and store them
            rb0 = 0
            for (j, owner_name) in enumerate(owner_names):

                # Define and draw a radio button
                io_assign = io_assign_dict[owner_name]
                rb1 = SingleRadioButton("", rb0, 1 if k in io_assign else 0)
                g2.setField(rb1, j, displayrow, padding=(0, 0, 0, 0),
                            anchorLeft=1)

                # Associate the radio button with an owner
                self.rbdicts[owner_name].update({k: rb1})
                rb0 = rb1

            # Slicing the string if it's too long, and replace with ...
            l = Label(v)
            if len(v) > _MAX_TEXT_DESC:
                s = v
                l = Label(s[0:_MAX_TEXT_DESC - 3] + "...")

            # Display the DRC and Description fields
            g2.setField(Label(k), num_owners, displayrow,
                        padding=(0, 0, 1, 0), anchorLeft=1)
            g2.setField(l, num_owners + 1, displayrow,
                        padding=(0, 0, 1, 0), anchorLeft=1)

            displayrow += 1

        # Drew: 1 - 8 of 15
        g3 = Grid(1, 2)
        g3.setField(Label(_("%d - %d of %d") %
                          (ioinfo.start + 1, ioinfo.end,
                           len(adapterdict))), 0, 0, padding=(0, 0, 0, 0),
                    anchorLeft=1)
        g3.setField(Label("_" * _BOTTOM_DASH_LEN), 0, 1,
                    padding=(0, 0, 0, 0), anchorLeft=1)

        # Need by super class, WizardPanel
        self.buttonBar = \
            IOWizardPanel.buildButtonBar(screen, len(adapterdict),
                                         ioinfo.start, ioinfo.end)
        # The screen grid-form has 4 parts
        gf.add(tb, 0, 0, padding=(0, 0, 1, 1), growx=1, anchorLeft=1)
        gf.add(g2, 0, 1, padding=(0, 1, 0, 0), growx=1)
        gf.add(g3, 0, 2, padding=(0, 1, 0, 0), growx=1)
        gf.add(self.buttonBar, 0, 3, padding=(0, 1, 0, 0), growx=1)

        # Keep an local copy of the overall adapter dictionary
        self.adapterdict = adapterdict

        super(IOWizardPanel, self).__init__(screen=screen,
                                            name=name, gridform=gf,
                                            panelaction=None)

    @staticmethod
    def buildButtonBar(screen, max_rows, start, end):
        # Build the button bar accordingly base on where we are
        # and how many physical adapters there are
        if max_rows <= IOWizardPanel.MAX_ENTRY:
            iopbb = ButtonBar(screen, ((_("Back"), WizardPanel.BACK),
                                       (_("Next"), WizardPanel.NEXT),
                                       (_("Cancel"), WizardPanel.CANCEL)),
                              compact=1)
        elif start == 0:
            iopbb = ButtonBar(screen, ((_("View More"), WizardPanel.MORE),
                                       (_("Back"), WizardPanel.BACK),
                                       (_("Next"), WizardPanel.NEXT),
                                       (_("Cancel"), WizardPanel.CANCEL)),
                              compact=1)
        elif end < max_rows:
            iopbb = ButtonBar(screen, ((_("View Previous"), WizardPanel.PREV),
                                       (_("View More"), WizardPanel.MORE),
                                       (_("Back"), WizardPanel.BACK),
                                       (_("Next"), WizardPanel.NEXT),
                                       (_("Cancel"), WizardPanel.CANCEL)),
                              compact=1)
        else:
            iopbb = ButtonBar(screen, ((_("View Previous"), WizardPanel.PREV),
                                       (_("Back"), WizardPanel.BACK),
                                       (_("Next"), WizardPanel.NEXT),
                                       (_("Cancel"), WizardPanel.CANCEL)),
                              compact=1)
        return iopbb

    def getValues(self):
        '''
        Returns a dictionary structure with SDN1, SDN2, VIOS1
        and VIOS2 as key. Each will have a list of drc names
        '''
        out_io_assign_dict = {}

        for owner, rbdict in self.rbdicts.items():

            # Convert a radio-button-selection mask vector to
            # a dictionary of the chosen adapters
            sub_iodict = {}
            for drc, rb in rbdict.items():
                if rb.selected():
                    sub_iodict.update({drc: self.adapterdict[drc]})

            out_io_assign_dict.update({owner: sub_iodict})

        return out_io_assign_dict
