# IBM CONFIDENTIAL
#
# OBJECT CODE ONLY SOURCE MATERIALS
#
# COPYRIGHT International Business Machines Corp. 2015
# All Rights Reserved
#
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.

import ast
from gettext import gettext as _
from snack import Entry
from snack import FLAG_DISABLED
from snack import FLAGS_SET
from snack import FLAGS_RESET
from snack import Grid
from snack import GridForm
from snack import Label
from snack import RadioGroup
from snack import TextboxReflowed
from network.network import Network
from network.network import BadNetworkSettingsException
from wizpkg.wizardpanel import WizardPanel
from wizpkg.extendedwizardpanels import buildButtonBar


class NetworkConfigWizardPanel(WizardPanel):
    '''
    This class extends the base WizardPanel and uses
    one or more Entry as widget in its gridform
    User will be allowed to enter values into the entry
    fields.

             +-------------------------------+
             |     some title                |
             +-------------------------------+
             | Here are some instructions..  |
             | o Static  o DHCP4  o DHCP6    |
             | Host Name   _________________ |
             | Domain Name _________________ |
             | Static IP   _________________ |
             | IP address  _________________ |
             | Gateway     _________________ |
             | DNS 1       _________________ |
             | DNS 2       _________________ |
             | ....                          |
             |                               |
             | <back>   <next>   <cancel>    |
             +-------------------------------+
    '''

    def __init__(self, screen, name, paneltitle, msg, timeobj, networkobj,
                 index, panelaction):
        '''
        Init method
        :param screen: Snackscreen instance
        :param paneltitile: Title of the panel
        :param msg: Instructions to user
        :param timeobj: Object containing time information
        :param networkobj: Object containing network information
        :param index: Indication of first or last panel in wizard
        :param panelaction: Function to call once all data has been entered
        '''
        gf = GridForm(screen, paneltitle, 2, 6)
        tb = TextboxReflowed(WizardPanel.WIDTH, msg)
        rg = RadioGroup()
        self.staticrb = rg.add(_("Static IP"), 'STATIC', 1
                               if networkobj.dhcp_protocol != Network.IPV4 and
                               networkobj.dhcp_protocol != Network.IPV6
                               else 0)
        self.dhcp4rb = rg.add(_("DHCP IPV4"), 'DHCP4', 1
                              if networkobj.dhcp_protocol == Network.IPV4
                              else 0)
        self.dhcp6rb = rg.add(_("DHCP IPV6"), 'DHCP6', 1
                              if networkobj.dhcp_protocol == Network.IPV6
                              else 0)

        # Save entry in separate Grid and get a handle to it
        # to clear data when transitioning to previous panel
        self.hostname = Entry(30, text=networkobj.hostname, password=0)
        self.domainname = Entry(30, text=networkobj.domainname, password=0)
        self.ipaddr = Entry(30, text=networkobj.ipaddress, password=0)
        self.gateway = Entry(30, text=networkobj.gateway, password=0)
        self.netmask = Entry(30, text=networkobj.networkmask, password=0)
        self.dns1 = Entry(30, text=networkobj.dns1, password=0)
        self.dns2 = Entry(30, text=networkobj.dns2, password=0)
        self.dns3 = Entry(30, text=networkobj.dns3, password=0)

        g = Grid(2, 10)
        rbg = Grid(3, 2)
        rbg.setField(self.staticrb, 0, 1, padding=(1, 0, 1, 0), anchorLeft=1)
        rbg.setField(self.dhcp4rb, 1, 1, padding=(4, 0, 1, 0), anchorLeft=1)
        rbg.setField(self.dhcp6rb, 2, 1, padding=(4, 0, 1, 0), anchorLeft=1)
        g.setField(Label(_("Host Name")), 0, 0, padding=(1, 0, 1, 0),
                   anchorLeft=1)
        g.setField(self.hostname, 1, 0, anchorLeft=1)
        g.setField(Label(_("Domain Name")), 0, 1, padding=(1, 0, 1, 0),
                   anchorLeft=1)
        g.setField(self.domainname, 1, 1, anchorLeft=1)
        g.setField(Label(_("IP Address")), 0, 2, padding=(1, 0, 1, 0),
                   anchorLeft=1)
        g.setField(self.ipaddr, 1, 2, anchorLeft=1)
        g.setField(Label(_("Gateway")), 0, 3, padding=(1, 0, 1, 0),
                   anchorLeft=1)
        g.setField(self.gateway, 1, 3, anchorLeft=1)
        g.setField(Label(_("Network Mask")), 0, 4, padding=(1, 0, 1, 0),
                   anchorLeft=1)
        g.setField(self.netmask, 1, 4, anchorLeft=1)
        g.setField(Label(_("DNS 1")), 0, 5, padding=(1, 0, 1, 0),
                   anchorLeft=1)
        g.setField(self.dns1, 1, 5, anchorLeft=1)
        g.setField(Label(_("DNS 2")), 0, 6, padding=(1, 0, 1, 0),
                   anchorLeft=1)
        g.setField(self.dns2, 1, 6, anchorLeft=1)
        g.setField(Label(_("DNS 3")), 0, 7, padding=(1, 0, 1, 0),
                   anchorLeft=1)
        g.setField(self.dns3, 1, 7, anchorLeft=1)
        if timeobj:
            self.ntpserver = Entry(30, text=timeobj.ntpserver, password=0)
            if ast.literal_eval(timeobj.ntpenabled):
                g.setField(Label(_("NTP Server")), 0, 8, padding=(1, 0, 1, 0),
                           anchorLeft=1)
                g.setField(self.ntpserver, 1, 8, anchorLeft=1)
        else:
            self.ntpserver = None
        # Adding widgets to GridForm. First and last widgets
        # will have padding at bottom and top respectively
        # to provide nice spacing.
        gf.add(tb, 0, 0, padding=(0, 0, 0, 1), anchorTop=0)
        gf.add(rbg, 0, 1, padding=(0, 0, 1, 0), anchorLeft=1, anchorTop=1)
        gf.add(g, 0, 2, growy=1, padding=(1, 1, 0, 0), anchorLeft=1,
               anchorTop=0)
        bb = buildButtonBar(screen, index)
        gf.add(bb, 0, 3, padding=(0, 1, 0, 0), growx=1)
        self.staticrb.setCallback(self.enableDisableStaticIP)
        self.dhcp4rb.setCallback(self.enableDisableStaticIP)
        self.dhcp6rb.setCallback(self.enableDisableStaticIP)
        self.buttonBar = bb
        self.enableDisableStaticIP()
        super(NetworkConfigWizardPanel, self).__init__(screen=screen,
                                                       name=name, gridform=gf,
                                                       panelaction=panelaction)

    def enableDisableStaticIP(self):
        '''
        Enable and disable static ip base on value of DHCP buttons
        '''
        s = FLAGS_SET if not self.staticrb.selected() else FLAGS_RESET
        self.ipaddr.setFlags(FLAG_DISABLED, s)
        self.gateway.setFlags(FLAG_DISABLED, s)
        self.netmask.setFlags(FLAG_DISABLED, s)

    def dhcpEnabled(self):
        '''
        Returns whether or not DHCP is desired
        '''
        return self.dhcp4rb.selected() or self.dhcp6rb.selected()

    def getValues(self):
        '''
        Return information in the form of a list
        '''
        entryList = []
        if self.dhcpEnabled():
            entryList.append(Network.IPV4 if self.dhcp4rb.selected()
                             else Network.IPV6)
        else:
            entryList.append("DISABLED")
        entryList.append(None if not self.hostname.value()
                         else self.hostname.value())
        entryList.append(None if not self.domainname.value()
                         else self.domainname.value())
        entryList.append(None if not self.ipaddr.value()
                         else self.ipaddr.value())
        entryList.append(None if not self.gateway.value()
                         else self.gateway.value())
        entryList.append(None if not self.netmask.value()
                         else self.netmask.value())
        entryList.append(None if not self.dns1.value()
                         else self.dns1.value())
        entryList.append(None if not self.dns2.value()
                         else self.dns2.value())
        entryList.append(None if not self.dns3.value()
                         else self.dns3.value())
        if self.ntpserver:
            entryList.append(None if not self.ntpserver.value()
                             else self.ntpserver.value())
        return entryList

    def validate(self):
        '''
        This method validate user input on this wizard panel.
        '''
        if self.dhcpEnabled():
            dhcpproto = (Network.IPV4 if self.dhcp4rb.selected() else
                         Network.IPV6)
        else:
            dhcpproto = ''
        networkobj = Network(self.hostname.value(), self.domainname.value(),
                             self.ipaddr.value(), self.netmask.value(),
                             self.gateway.value(), dhcpproto,
                             self.dns1.value(), self.dns2.value(),
                             self.dns3.value())
        try:
            Network.validate(networkobj)
        except BadNetworkSettingsException as e:
            WizardPanel.showError(self.screen, str(e))
            return 1
        return 0
