# IBM CONFIDENTIAL
#
# OBJECT CODE ONLY SOURCE MATERIALS
#
# COPYRIGHT International Business Machines Corp. 2015
# All Rights Reserved
#
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.
from gettext import gettext as _
from wizpkg.wizardpanel import WizardPanel
from wizpkg.extendedwizardpanels import RadioButtonWizardAdvancePanel
from wizpkg.extendedwizardpanels import MissingTupleException
from ioadapter.iopanel import natural_order


class NetworkBridgePanel(object):
    '''
    This class represents the display panel that is used by the installation
    operator to configure a Virtual Network Bridge.
    '''
    FIRSTONLY = "firstonly"
    FIRSTPAGE = "first"
    MIDPAGE = "middle"
    LASTPAGE = "last"
    BRIDGE_NAME = "bridge_name"
    BRIDGE_PVID_SHOW = "bridge_pvid_show"
    BRIDGE_PVID_EDIT = "bridge_pvid_edit"
    BRIDGE_PVID = "bridge_pvid"

    def __init__(self, ntwk_ports_info, persistedinfo, bridge_info):
        '''
        Init method
        :param ntwk_ports_info: A dictionary of all available Network ports.
        :param persistedinfo: A tuple of previously selected network ports.
        :param bridge_info: A dictionary of bridge-specific information
        '''

        self.bridge_info = bridge_info
        self.num_ports = len(ntwk_ports_info)
        port_loc_codes = sorted(ntwk_ports_info.keys(), key=natural_order)
        self.pagelist = []
        buttonlist = []
        for port_loc in port_loc_codes:
            port_num = port_loc[::-1].split('-', 1)[0][:-1][::-1]
            adapter_loc = port_loc[::-1].split('-', 1)[1][::-1]

            if (len(buttonlist) == 4):
                # Add a new page
                self.pagelist.append(buttonlist)
                buttonlist = []

            if port_loc in persistedinfo:
                pre_select_this = 1
            else:
                pre_select_this = 0
            buttonlist.append((port_num, adapter_loc,
                               ntwk_ports_info[port_loc][:33],
                               port_loc, pre_select_this))

        self.pagelist.append(buttonlist)
        self.num_pages = len(self.pagelist)

    def run_networkbridge(self, screen):
        '''
        The networkbridge Main function.
        '''

        # Start with the first page and empty list
        page_num = 0
        returnlist = []

        while True:
            # Get number of adapters in the page out of the total adapters
            submsg = _("%d - %d of %d")
            pagevalue = (submsg %
                         (4 * page_num + 1, 4 * page_num +
                          len(self.pagelist[page_num]), self.num_ports))
            # Execute Panel(s)
            if (page_num == 0):
                if(self.num_pages == 1):
                    page_loc = NetworkBridgePanel.FIRSTONLY
                else:
                    page_loc = NetworkBridgePanel.FIRSTPAGE
            elif (page_num < (self.num_pages - 1)):
                page_loc = NetworkBridgePanel.MIDPAGE
            else:
                page_loc = NetworkBridgePanel.LASTPAGE

            (nextpanel, selectedlist, pvid) = self.show_network_screen(
                screen, self.pagelist[page_num], pagevalue, page_loc)

            # Remember pvid from user selection if any
            self.bridge_info[NetworkBridgePanel.BRIDGE_PVID] = pvid
            # Save selected to the original page list
            for index, port_info in enumerate(self.pagelist[page_num]):
                (t1, t2, t3, port_loc, sel) = port_info
                if (selectedlist and (port_loc in selectedlist)):
                    sel = 1
                else:
                    sel = 0
                self.pagelist[page_num][index] = (t1, t2, t3, port_loc, sel)

            # Execute Panel Actions
            if nextpanel == WizardPanel.CANCEL:
                break
            elif nextpanel == WizardPanel.PREV:
                # Go back a page
                page_num -= 1
            elif nextpanel == WizardPanel.MORE:
                # Move to the next page
                page_num += 1
            elif nextpanel == WizardPanel.BACK:
                break
            elif nextpanel == WizardPanel.NEXT:
                returnlist = self.determine_selected_ports(screen, pvid)
                if (len(returnlist) <= 0):
                    # An error occurred
                    continue
                else:
                    break

        return (nextpanel, returnlist, pvid)

    def determine_selected_ports(self, screen, pvid):
        returnlist = []
        pvid_is_valid = True
        pvid_show = self.bridge_info[NetworkBridgePanel.BRIDGE_PVID_SHOW]
        try:
            int(pvid)
        except ValueError:
            pvid_is_valid = False
        if (pvid_show and (not pvid_is_valid)):
            WizardPanel.showError(screen, _("PVID must be an integer value."))
            return (returnlist)

        # Iterate through the button pages and
        # save the selected adapters
        for page in self.pagelist:
            for row in page:
                val = row[3]
                sel = row[4]
                if sel == 1:
                    returnlist.append(val)
        # Cannot have an empty list
        if (len(returnlist) <= 0):
            WizardPanel.showError(screen,
                                  _("At least one adapter "
                                    "needs to be selected.\n"
                                    "Please select an Adapter."))

        return (returnlist)

    def show_network_screen(self, screen, panelButtons, panelPage, paneltype):
        panelName = _("Network Panel")
        panelTitle = _("Virtual Network Bridges")
        bridge_name = self.bridge_info[NetworkBridgePanel.BRIDGE_NAME]
        pvid_show = self.bridge_info[NetworkBridgePanel.BRIDGE_PVID_SHOW]
        pvid_edit = self.bridge_info[NetworkBridgePanel.BRIDGE_PVID_EDIT]
        pvid = str(self.bridge_info[NetworkBridgePanel.BRIDGE_PVID])
        panelMessage = {}
        bridge_display_name = {}
        panelMessage["VIOS1"] = panelMessage["VIOS2"] = \
            _("Select one port to bind each Shared Ethernet Adapter (SEA) "
              "to a physical network. Select 2 or more "
              "ports to create link aggregation")
        panelMessage["SDN1"] = panelMessage["SDN2"] = \
            _("Select one port to bind the Software Defined Network"
              " (SDN) Bridge to a physical network. Select 2 or more "
              "ports to create link aggregation")
        bridge_display_name["SDN1"] = bridge_display_name["SDN2"] =\
            _("Network Bridge: SDN")
        bridge_display_name["VIOS1"] = _("Network Bridge: VIOS1")
        bridge_display_name["VIOS2"] = _("Network Bridge: VIOS2")
        if (pvid_show):
            panelDisplaylist = (bridge_display_name[bridge_name],
                                _("Port VLAN ID: "), pvid)
        else:
            panelDisplaylist = (bridge_display_name[bridge_name],)
        panelTitlelist = (_("Port"), _("Location Code"),
                          _("Adapter Description"))

        try:
            networkPanel = RadioButtonWizardAdvancePanel(
                screen, panelName, panelTitle, panelMessage[bridge_name],
                panelDisplaylist, panelTitlelist, panelButtons,
                panelPage, paneltype, 3, pvid_edit, panelaction=None)
            networkPanel.run()
        except MissingTupleException as e:
            WizardPanel.showError(screen, _(str(e)))

        pvid = ''
        if (pvid_show):
            pvid = networkPanel.getTextValue()
        return (networkPanel.next(), networkPanel.getValues(),
                pvid)
