#!/usr/bin/python3
# IBM CONFIDENTIAL
#
# OBJECT CODE ONLY SOURCE MATERIALS
#
# COPYRIGHT International Business Machines Corp. 2015
# All Rights Reserved
#
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.

from gettext import gettext as _
from configparser import ConfigParser
from configparser import ExtendedInterpolation
from configparser import RawConfigParser
from configparser import NoSectionError
from configparser import NoOptionError
from user.user import User
from user.user import BadUserNameException
from network.network import Network
from network.network import BadNetworkSettingsException
from procmem.procunit import BadProcUnitException
from procmem.procunit import ProcUnitInfo
from procmem.virtprocinfo import BadVirtProcSettingsException
from procmem.virtprocinfo import VirtProcInfo
from procmem.meminfo import BadMemorySettingsException
from procmem.meminfo import MemInfo
from diskselect.targetdiskinstall import TargetDisk
from diskselect.targetdiskinstall import StorageDevicesException
import ast
import getopt
import gettext
import ipaddress
import locale
import six
import sys


class PVMConfigData(object):
    '''
    This class provides getter/setter/validator methods
    for the PVM Install configuration file. The configuration
    file is provided in the following ways:
    - Edited by the users and specified as an argument to the kernel
      in the grub.cfg file on a network boot.
    - Provided on removable media with default settings.
    Upon boot, information will be gathered and filled in, for example
    physical i/o slots on the system.
    get and set methods in this class handle read/update of attributes
    defined in the various sections.
    A main is provided to dump content of config data as well as use as
    unit test.
    '''
    '''
    Definition of Section
    '''
    SYSCONFIG_SECTION = 'SystemConfig'
    GENERAL_SETTINGS_SECTION = 'NovaLinkGeneralSettings'
    NETWORK_SETTINGS_SECTION = 'NovaLinkNetworkSettings'
    APT_REPO_SETTINGS_SECTION = 'NovaLinkAPTRepoConfig'
    CREDENTIALS_SECTION = 'NovaLinkAdminCredentials'
    PVM_CONFIG_SECTION = 'NovaLinkConfiguration'
    PVMSDN_CONFIG_SECTION = 'NovaLinkSDNConfiguration'
    SDN_CONFIG_SECTION = 'SDNConfiguration'
    VIOS_CONFIG_SECTION = 'VIOSConfiguration'
    VIOSNIMSERVER_CONFIG_SECTION = 'VIOSNIMServerConfig'
    VIOS_CREDENTIALS_SECTION = 'VIOSAdminCredentials'
    VIOS1_PHYS_IO_CONFIG_SECTION = 'VIOS1PhysicalIOConfig'
    VIOS2_PHYS_IO_CONFIG_SECTION = 'VIOS2PhysicalIOConfig'
    SDN1_PHYS_IO_CONFIG_SECTION = 'SDN1PhysicalIOConfig'
    SDN2_PHYS_IO_CONFIG_SECTION = 'SDN2PhysicalIOConfig'
    VIOS1_NETW_SETTINGS_SECTION = 'VIOS1NetworkSettings'
    VIOS2_NETW_SETTINGS_SECTION = 'VIOS2NetworkSettings'
    VIOS1_NETWORK_BRIDGE = 'VIOS1NetworkBridge'
    VIOS2_NETWORK_BRIDGE = 'VIOS2NetworkBridge'
    SDN1_NETWORK_BRIDGE = 'SDN1NetworkBridge'
    SDN2_NETWORK_BRIDGE = 'SDN2NetworkBridge'

    '''
    Individual Attribute name
    '''
    DHCPIP = 'DHCPIP'
    IPADDR = 'IPAddress'
    GATEWAY = 'Gateway'
    NETMASK = 'NetMask'
    DNS1 = 'DNS1'
    DNS2 = 'DNS2'
    DNS3 = 'DNS3'
    HOSTNAME = 'HostName'
    DOMAIN = 'Domain'
    NAME = 'Name'
    PROCTYPE = 'ProcType'
    SHAREDMODE = 'SharedMode'
    UNCAPPEDWEIGHT = 'UncappedWeight'
    PROCUNITMIN = 'ProcUnitMin'
    PROCUNITMAX = 'ProcUnitMax'
    PROCUNITDESIRED = 'ProcUnitDesired'
    VIRTUALPROCMIN = 'VirtProcMin'
    VIRTUALPROCMAX = 'VirtProcMax'
    VIRTUALPROCDESIRED = 'VirtProcDesired'
    IOREDUNDANCY = 'IORedundancy'
    SDN_ENABLED = "SDNEnabled"
    SDN_REDUN_ENABLED = "SDNRedunEnabled"
    SDN_REDUN_SEL_ENABLED = "SDNRedunSelEnabled"
    DESIREDMEMORY = 'DesiredMemory'
    MINMEMORY = 'MinMemory'
    MAXMEMORY = 'MaxMemory'
    MAXVIRTSLOTS = 'MaxVirtualSlots'
    PHYSICALIOSLOT = 'PhysicalIOSlot'
    NTPENABLED = 'NTPEnabled'
    NTPSERVER = 'NTPServer'
    TIMEZONE = 'TimeZone'
    DOWNLOADPROTO = 'DownloadProtocol'
    MIRRORHOSTNAME = 'MirrorHostName'
    MIRRORDIRECTORY = 'MirrorDirectory'
    MIRRORPROXY = 'MirrorProxy'
    USERNAME = 'UserName'
    PASSWORD = 'Password'
    USERDESC = 'UserDescription'
    SERIALNUMBER = 'SerialNumber'
    LMBSIZE = 'LMBSize'
    INFO = 'Info'
    TOTALMEMORY = 'TotalMemory'
    NOVALINKPRIVATEIPV4 = 'NOVALINK_PRIVATE_IP'
    VIOS1PRIVATEIPV4 = 'VIOS1_PRIVATE_IP'
    VIOS2PRIVATEIPV4 = 'VIOS2_PRIVATE_IP'
    PRIVATENETMASK = 'NOVALINK_NETMASK'
    VIOSINSTALLPROMPT = 'VIOSInstallPrompt'
    LINK_AGGREGATION_TYPE = 'LinkAggregationType'
    FAILURE_DETECTION_ALGORITHM = 'FailureDetectionAlgorithm'
    PVID = 'PVID'
    INSTALL_OTHER_OS = 'InstallOtherOS'
    INSTALLMODE = 'InstallMode'
    SDE = 'SDE'
    SDN = 'SDN'
    TARGET_DISK_INSTALL = 'TargetDiskInstall'
    TARGET_NETWORK_DEVICE = 'TargetNetworkDevice'
    POST_INSTALL_COMMAND = 'PostInstallCommand'
    OTHER_OS_NAME = 'OSName'
    KS_FILE = 'KickstartFile'
    INSTALLATION_SOURCE = 'InstallationSource'

    SYSTEM_ATTRS = (SERIALNUMBER, LMBSIZE)
    NETWORK_ATTRS = (DHCPIP, IPADDR, GATEWAY, NETMASK, DNS1, DNS2, DNS3,
                     HOSTNAME, DOMAIN)
    VIOS_CONFIG_ATTRS = (MINMEMORY, MAXMEMORY, DESIREDMEMORY,
                         PROCUNITMIN, PROCUNITMAX, PROCUNITDESIRED,
                         VIRTUALPROCMIN, VIRTUALPROCMAX, VIRTUALPROCDESIRED,
                         PROCTYPE, SHAREDMODE, UNCAPPEDWEIGHT, MAXVIRTSLOTS)
    CONFIG_ATTRS = (NAME, PROCTYPE, SHAREDMODE, UNCAPPEDWEIGHT,
                    PROCUNITMIN, PROCUNITMAX, PROCUNITDESIRED,
                    VIRTUALPROCMIN, VIRTUALPROCMAX, VIRTUALPROCDESIRED,
                    MINMEMORY, MAXMEMORY, DESIREDMEMORY, IOREDUNDANCY,
                    MAXVIRTSLOTS, INSTALLMODE, TARGET_DISK_INSTALL,
                    TARGET_NETWORK_DEVICE, POST_INSTALL_COMMAND,
                    INSTALL_OTHER_OS, OTHER_OS_NAME, KS_FILE,
                    INSTALLATION_SOURCE)
    GENERAL_ATTRS = (NTPENABLED, NTPSERVER, TIMEZONE)
    APT_REPO_ATTRS = (DOWNLOADPROTO, MIRRORHOSTNAME, MIRRORDIRECTORY,
                      MIRRORPROXY)
    USER_ATTRS = (USERNAME, PASSWORD, USERDESC)
    VIOS1_NETWORK_ATTRS = (DHCPIP, IPADDR, GATEWAY, NETMASK, DNS1, DNS2, DNS3,
                           HOSTNAME, DOMAIN)
    VIOS2_NETWORK_ATTRS = (DHCPIP, IPADDR, GATEWAY, NETMASK, DNS1, DNS2, DNS3,
                           HOSTNAME, DOMAIN)
    NIM_ATTRS = (NOVALINKPRIVATEIPV4, VIOS1PRIVATEIPV4, VIOS2PRIVATEIPV4,
                 PRIVATENETMASK, VIOSINSTALLPROMPT)
    SDN_CONFIG_ATTRS = (SDN_ENABLED, SDN_REDUN_ENABLED, SDN_REDUN_SEL_ENABLED)

    def __init__(self, configfile, raw=False):
        try:

            # Now use ConfigParser to obtain the needed information
            # Another implementation is to just parse stdout but
            # using a file allows us to later save it so we can debug
            # on reboot. For case where attribute value can have special
            # charater such as '$' use raw mode to avoid confusing
            # interpolation trying to evaluate variables

            self.configFile = configfile
            if raw:
                self.config = RawConfigParser()
            else:
                self.config = ConfigParser(interpolation=ExtendedInterpolation
                                           ())
            self.config.read(configfile)
        except:
            raise ValueError(_("Unable to read configuration file"))

    def _get_pvmconfig_attribute(self, attribute):
        '''
        Return the given attribute's value from either the
        [NovaLinkConfiguration] section or the [NovaLinkSDNConfiguration]
        section depending on whether SDN is disabled or enabled.  When the
        attribute is present in neither section, return an empty string.
        '''
        tmp_str = ''
        try:
            # If SDN is enabled then
            #   get the attribute from PVMSDN_CONFIG_SECTION
            # else
            #   get the attribute from PVM_CONFIG_SECTION
            tmp_str = self.config.get(self.SDN_CONFIG_SECTION,
                                      self.SDN_ENABLED)
            sdn_enabled = ast.literal_eval(str(tmp_str))
        except (NoSectionError, KeyError, NoOptionError):
            sdn_enabled = False

        if not sdn_enabled:
            try:
                tmp_str = self.config.get(self.PVM_CONFIG_SECTION, attribute)
            except (NoSectionError, KeyError, NoOptionError):
                tmp_str = ''
            return tmp_str

        # sdn_enabled == True
        try:
            tmp_str = self.config.get(self.PVMSDN_CONFIG_SECTION,
                                      attribute)
        except (NoSectionError, KeyError, NoOptionError):
            try:
                tmp_str = self.config.get(self.PVM_CONFIG_SECTION,
                                          attribute)
            except (NoSectionError, KeyError, NoOptionError):
                tmp_str = ''

        return tmp_str

    def get_attribute(self, section, attribute):
        '''
        Obtain attribute given a section
        If section does not exist an empty string
        will be returned. This is done to keep compatibility with
        older version of the configuration file who does not have
        new Section/Attribute added in new release
        '''

        if section == self.PVM_CONFIG_SECTION:
            return self._get_pvmconfig_attribute(attribute)

        tmp_str = ''
        try:
            tmp_str = self.config.get(section, attribute)
        except (NoSectionError, KeyError, NoOptionError):
            return ''
        return tmp_str

    def get_attribute_boolean(self, sec, attribute, defaultval):
        '''
        Handle Section or attribute that does not exist
        and return the given default value
        '''

        if (sec == self.PVM_CONFIG_SECTION):
            tmp_str = self._get_pvmconfig_attribute(attribute)
        else:
            try:
                tmp_str = self.config.get(sec, attribute)
            except (NoSectionError, KeyError, NoOptionError):
                return defaultval

        rv = ast.literal_eval(str(tmp_str))
        if (not type(rv) is bool):
            return (defaultval)
        else:
            return (rv)

    def get_attribute_notstr(self, sec, attribute):
        '''
        Handle Section or attribute that does not exist
        and return empty string
        '''

        if (sec == self.PVM_CONFIG_SECTION):
            tmp_str = self._get_pvmconfig_attribute(attribute)
        else:
            try:
                tmp_str = self.config.get(sec, attribute)
            except (NoSectionError, KeyError, NoOptionError):
                return ''
        return ast.literal_eval(str(tmp_str))

    def _set_pvmconfig_attribute(self, attribute, value):
        '''
        Set the given attribute's value in either the [NovaLinkConfiguration]
        section of the [NovaLinkSDNConfiguration] section depending on whether
        SDN is disabled or enabled.
        '''
        tmp_str = ''
        try:
            # If SDN is enabled then
            #   get the attribute from PVMSDN_CONFIG_SECTION
            # else
            #   get the attribute from PVM_CONFIG_SECTION
            tmp_str = self.config.get(self.SDN_CONFIG_SECTION,
                                      self.SDN_ENABLED)
            sdn_enabled = ast.literal_eval(str(tmp_str))
        except (NoSectionError, KeyError, NoOptionError):
            sdn_enabled = False

        if sdn_enabled:
            section = self.PVMSDN_CONFIG_SECTION
            # First make sure that the given attribute exists in
            # PVMSDN_CONFIG_SECTION.  If it does not exist, fall back to
            # PVM_CONFIG_SECTION
            try:
                tmp_str = self.config.get(section, attribute)
            except (NoSectionError, KeyError, NoOptionError):
                section = self.PVM_CONFIG_SECTION
        else:
            section = self.PVM_CONFIG_SECTION

        self.config.set(section, attribute, value)
        self.config.write(open(self.configFile, 'w'))

    def set_attribute(self, section, attribute, val):
        '''
        Set the value of the given attribute in a given section
        '''
        if section == self.PVM_CONFIG_SECTION:
            self._set_pvmconfig_attribute(attribute, val)
        else:
            self.config.set(section, attribute, val)
            self.config.write(open(self.configFile, 'w'))


def printusage():
    six.print_(_("Usage: %s -f file [-r] -o d | v | m [-a <sect>:<attr> -v "
                 "<val>] -s section | ALL [-h | --help]") % sys.argv[0],
               file=sys.stderr)
    six.print_(_("     -f    : Configuration file"), file=sys.stderr)
    six.print_(_("     -r    : Use raw mode"), file=sys.stderr)
    six.print_(_("     -s    : Section to dump. ALL will dump all sections."),
               file=sys.stderr)
    six.print_(_("     -o    : Operation to perform. d for dumping file "
                 "content,"),
               file=sys.stderr)
    six.print_(_("             combine with -a to dump individual attribute"),
               file=sys.stderr)
    six.print_(_("           : v for validation and m to modify an "
                 "attribute."),
               file=sys.stderr)
    six.print_(_("     -a    : Name of attribute. Must be of the form"),
               file=sys.stderr)
    six.print_(_("           : <section>:<attribute name>"), file=sys.stderr)
    six.print_(_("     -v    : Attribute value to set"), file=sys.stderr)
    six.print_(_("-h, --help : Prints this help message"), file=sys.stderr)
    sys.exit(2)


def check_duplicate_entries(cfgp, greenfield_mode, redundant_vios, inst_mode):
    '''
    Check duplicate attribute in the file. Rules listed below
    - Duplicate hostname and ip addresses are not valid if
      greenfield mode is True. Second VIOS information will be
      checked if redundant_vios is set to True
    '''
    rcval = 0
    hn1 = cfgp.get_attribute(PVMConfigData.NETWORK_SETTINGS_SECTION,
                             PVMConfigData.HOSTNAME)
    ip1 = cfgp.get_attribute(PVMConfigData.NETWORK_SETTINGS_SECTION,
                             PVMConfigData.IPADDR)
    if greenfield_mode and inst_mode != PVMConfigData.SDE:
        hn2 = cfgp.get_attribute(PVMConfigData.VIOS1_NETW_SETTINGS_SECTION,
                                 PVMConfigData.HOSTNAME)
        ip2 = cfgp.get_attribute(PVMConfigData.VIOS1_NETW_SETTINGS_SECTION,
                                 PVMConfigData.IPADDR)
        if hn2 == hn1:
            six.print_(_("Duplicate hostname found in sections: %s %s")
                       % (PVMConfigData.NETWORK_SETTINGS_SECTION,
                          PVMConfigData.VIOS1_NETW_SETTINGS_SECTION),
                       file=sys.stderr)
            rcval = 1
        if ip2 == ip1:
            six.print_(_("Duplicate ip address found in sections: %s %s")
                       % (PVMConfigData.NETWORK_SETTINGS_SECTION,
                          PVMConfigData.VIOS1_NETW_SETTINGS_SECTION),
                       file=sys.stderr)
            rcval = 1
        if redundant_vios:
            hn2 = cfgp.get_attribute(PVMConfigData.VIOS2_NETW_SETTINGS_SECTION,
                                     PVMConfigData.HOSTNAME)
            ip2 = cfgp.get_attribute(PVMConfigData.VIOS2_NETW_SETTINGS_SECTION,
                                     PVMConfigData.IPADDR)
            if hn2 == hn1:
                six.print_(_("Duplicate hostname found in sections: %s %s")
                           % (PVMConfigData.NETWORK_SETTINGS_SECTION,
                              PVMConfigData.VIOS2_NETW_SETTINGS_SECTION),
                           file=sys.stderr)
                rcval = 1
            if ip2 == ip1:
                six.print_(_("Duplicate ip address found in sections: %s %s")
                           % (PVMConfigData.NETWORK_SETTINGS_SECTION,
                              PVMConfigData.VIOS2_NETW_SETTINGS_SECTION),
                           file=sys.stderr)
                rcval = 1
    return rcval


def dumpattr(cfg_file, attr_to_dump):
    split_attr = attr_to_dump.split(':', 1)
    print(cfg_file.get_attribute(split_attr[0], split_attr[1]))


def dumpcfg(cfp, section):
    if section == 'ALL' or section == PVMConfigData.SYSCONFIG_SECTION:
        for i in PVMConfigData.SYSTEM_ATTRS:
            val = cfp.get_attribute(PVMConfigData.SYSCONFIG_SECTION, i)
            print(i.upper() + "='" + val + "'")
    if section == 'ALL' or section == PVMConfigData.GENERAL_SETTINGS_SECTION:
        for i in PVMConfigData.GENERAL_ATTRS:
            val = cfp.get_attribute(PVMConfigData.GENERAL_SETTINGS_SECTION, i)
            print(i.upper() + "='" + val + "'")
    if section == 'ALL' or section == PVMConfigData.NETWORK_SETTINGS_SECTION:
        for i in PVMConfigData.NETWORK_ATTRS:
            val = cfp.get_attribute(PVMConfigData.NETWORK_SETTINGS_SECTION, i)
            print(i.upper() + "='" + val + "'")
    if section == 'ALL' or (section ==
                            PVMConfigData.VIOSNIMSERVER_CONFIG_SECTION):
        for i in PVMConfigData.NIM_ATTRS:
            val = cfp.get_attribute(PVMConfigData.VIOSNIMSERVER_CONFIG_SECTION,
                                    i)
            print(i.upper() + "='" + val + "'")

    if section == 'ALL' or (section ==
                            PVMConfigData.VIOS1_NETW_SETTINGS_SECTION):
        for i in PVMConfigData.VIOS1_NETWORK_ATTRS:
            val = cfp.get_attribute(PVMConfigData.VIOS1_NETW_SETTINGS_SECTION,
                                    i)
            print(i.upper() + "='" + val + "'")
    if section == 'ALL' or (section ==
                            PVMConfigData.VIOS2_NETW_SETTINGS_SECTION):
        for i in PVMConfigData.VIOS2_NETWORK_ATTRS:
            val = cfp.get_attribute(PVMConfigData.VIOS2_NETW_SETTINGS_SECTION,
                                    i)
            print(i.upper() + "='" + val + "'")
    if section == 'ALL' or section == PVMConfigData.CREDENTIALS_SECTION:
        for i in PVMConfigData.USER_ATTRS:
            val = cfp.get_attribute(PVMConfigData.CREDENTIALS_SECTION, i)
            print(i.upper() + "='" + val + "'")
    if section == 'ALL' or section == PVMConfigData.VIOS_CREDENTIALS_SECTION:
        val = cfp.get_attribute(PVMConfigData.VIOS_CREDENTIALS_SECTION,
                                PVMConfigData.PASSWORD)
        print(PVMConfigData.PASSWORD.upper() + "='" + val + "'")
    if section == 'ALL' or section == PVMConfigData.APT_REPO_SETTINGS_SECTION:
        for i in PVMConfigData.APT_REPO_ATTRS:
            val = cfp.get_attribute(PVMConfigData.APT_REPO_SETTINGS_SECTION, i)
            print(i.upper() + "='" + val + "'")
    if section == 'ALL' or section == PVMConfigData.PVM_CONFIG_SECTION:
        for i in PVMConfigData.CONFIG_ATTRS:
            val = cfp.get_attribute(PVMConfigData.PVM_CONFIG_SECTION, i)
            print(i.upper() + "='" + val + "'")
    if section == 'ALL' or section == PVMConfigData.VIOS_CONFIG_SECTION:
        for i in PVMConfigData.VIOS_CONFIG_ATTRS:
            val = cfp.get_attribute(PVMConfigData.VIOS_CONFIG_SECTION, i)
            print(i.upper() + "='" + val + "'")
    if section == 'ALL' or section == PVMConfigData.SDN_CONFIG_SECTION:
        for i in PVMConfigData.SDN_CONFIG_ATTRS:
            val = cfp.get_attribute(PVMConfigData.SDN_CONFIG_SECTION, i)
            print(i.upper() + "='" + val + "'")

    if section == 'ALL' or section == PVMConfigData.VIOS1_NETWORK_BRIDGE:
        val = cfp.get_attribute(PVMConfigData.VIOS1_NETWORK_BRIDGE,
                                PVMConfigData.INFO)
        print(PVMConfigData.INFO + "=" + val)
        val = cfp.get_attribute(PVMConfigData.VIOS1_NETWORK_BRIDGE,
                                PVMConfigData.LINK_AGGREGATION_TYPE)
        print(PVMConfigData.LINK_AGGREGATION_TYPE + "=" + val)
        val = cfp.get_attribute(PVMConfigData.VIOS1_NETWORK_BRIDGE,
                                PVMConfigData.PVID)
        print(PVMConfigData.PVID + "=" + val)
    if section == 'ALL' or section == PVMConfigData.VIOS2_NETWORK_BRIDGE:
        val = cfp.get_attribute(PVMConfigData.VIOS2_NETWORK_BRIDGE,
                                PVMConfigData.INFO)
        print(PVMConfigData.INFO + "=" + val)
        val = cfp.get_attribute(PVMConfigData.VIOS2_NETWORK_BRIDGE,
                                PVMConfigData.LINK_AGGREGATION_TYPE)
        print(PVMConfigData.LINK_AGGREGATION_TYPE + "=" + val)
    if section == 'ALL' or section == PVMConfigData.SDN1_NETWORK_BRIDGE:
        val = cfp.get_attribute(PVMConfigData.SDN1_NETWORK_BRIDGE,
                                PVMConfigData.INFO)
        print(PVMConfigData.INFO + "=" + val)
        val = cfp.get_attribute(PVMConfigData.SDN1_NETWORK_BRIDGE,
                                PVMConfigData.LINK_AGGREGATION_TYPE)
        print(PVMConfigData.LINK_AGGREGATION_TYPE + "=" + val)
        val = cfp.get_attribute(PVMConfigData.SDN1_NETWORK_BRIDGE,
                                PVMConfigData.FAILURE_DETECTION_ALGORITHM)
        print(PVMConfigData.FAILURE_DETECTION_ALGORITHM + "=" + val)
    if section == 'ALL' or section == PVMConfigData.SDN2_NETWORK_BRIDGE:
        val = cfp.get_attribute(PVMConfigData.SDN2_NETWORK_BRIDGE,
                                PVMConfigData.INFO)
        print(PVMConfigData.INFO + "=" + val)
        val = cfp.get_attribute(PVMConfigData.SDN2_NETWORK_BRIDGE,
                                PVMConfigData.LINK_AGGREGATION_TYPE)
        print(PVMConfigData.LINK_AGGREGATION_TYPE + "=" + val)
        val = cfp.get_attribute(PVMConfigData.SDN2_NETWORK_BRIDGE,
                                PVMConfigData.FAILURE_DETECTION_ALGORITHM)
        print(PVMConfigData.FAILURE_DETECTION_ALGORITHM + "=" + val)
    doPrint = (section == 'ALL' or section ==
               PVMConfigData.VIOS1_PHYS_IO_CONFIG_SECTION)
    if doPrint:
        val = cfp.get_attribute(PVMConfigData.VIOS1_PHYS_IO_CONFIG_SECTION,
                                PVMConfigData.INFO)
        if val:
            result = ast.literal_eval(val)
            strval = ""
            if isinstance(result, dict):
                for item in result.keys():
                    strval = strval + " " + item
            print(PVMConfigData.VIOS1_PHYS_IO_CONFIG_SECTION + "=" + strval)

    doPrint = (section == 'ALL' or section ==
               PVMConfigData.VIOS2_PHYS_IO_CONFIG_SECTION)
    if doPrint:
        val = cfp.get_attribute(PVMConfigData.VIOS2_PHYS_IO_CONFIG_SECTION,
                                PVMConfigData.INFO)
        if val:
            result = ast.literal_eval(val)
            strval = ""
            if isinstance(result, dict):
                for item in result.keys():
                    strval = strval + " " + item
            print(PVMConfigData.VIOS2_PHYS_IO_CONFIG_SECTION + "=" + strval)


def modify(cfp, attr, value):
    attrmod = attr.split(':', 1)
    sect = attrmod[0]
    attrname = attrmod[1]
    cfp.set_attribute(sect, attrname, value)


def _validate_sdn_configuration(cfp1):

    cfg_section = PVMConfigData.SDN_CONFIG_SECTION

    cf_list = [PVMConfigData.SDN_ENABLED,
               PVMConfigData.SDN_REDUN_ENABLED,
               PVMConfigData.SDN_REDUN_SEL_ENABLED]

    for cf in cf_list:
        valu = cfp1.get_attribute(cfg_section, cf)

        if valu:
            try:
                ast.literal_eval(valu)
            except ValueError as e:
                six.print_(_("Error found in %s section, field %s: %s") %
                           (cfg_section, cf, str(e)), file=sys.stderr)
                return 1

    return 0


def _validate_vios_install(cfp):

    nimserver_config = PVMConfigData.VIOSNIMSERVER_CONFIG_SECTION
    val = cfp.get_attribute(nimserver_config,
                            PVMConfigData.NOVALINKPRIVATEIPV4)
    if val:
        try:
            ipaddress.IPv4Network(val)
        except ValueError as e:
            six.print_(_("Error found in %s section, field %s: %s") %
                       (nimserver_config, PVMConfigData.NOVALINKPRIVATEIPV4,
                        str(e)), file=sys.stderr)
            return 1
    val = cfp.get_attribute(nimserver_config, PVMConfigData.VIOS1PRIVATEIPV4)
    if val:
        try:
            ipaddress.IPv4Network(val)
        except ValueError as e:
            six.print_(_("Error found in %s section, field %s: %s") %
                       (nimserver_config, PVMConfigData.VIOS1PRIVATEIPV4,
                        str(e)), file=sys.stderr)
            return 1
    val = cfp.get_attribute(nimserver_config, PVMConfigData.VIOS2PRIVATEIPV4)
    if val:
        try:
            ipaddress.IPv4Network(val)
        except ValueError as e:
            six.print_(_("Error found in %s section, field %s: %s") %
                       (nimserver_config, PVMConfigData.VIOS2PRIVATEIPV4,
                        str(e)), file=sys.stderr)
            return 1

    val = cfp.get_attribute(nimserver_config, PVMConfigData.PRIVATENETMASK)
    if val:
        try:
            ipaddress.IPv4Network(val)
        except ValueError as e:
            six.print_(_("Error found in %s section, field %s: %s") %
                       (nimserver_config, PVMConfigData.PRIVATENETMASK,
                        str(e)), file=sys.stderr)
            return 1

    val = cfp.get_attribute(nimserver_config, PVMConfigData.VIOSINSTALLPROMPT)
    if val:
        try:
            ast.literal_eval(val)
        except ValueError as e:
            six.print_(_("Error found in %s section, field %s: %s") %
                       (nimserver_config, PVMConfigData.VIOSINSTALLPROMPT,
                        str(e)), file=sys.stderr)
            return 1
    return 0


def _validate_user(cfp):
    val = cfp.get_attribute(PVMConfigData.CREDENTIALS_SECTION,
                            PVMConfigData.USERNAME)
    try:
        User.validateusername(val)
    except BadUserNameException as e:
        six.print_(_("Error found in %s section: %s") %
                   (PVMConfigData.CREDENTIALS_SECTION, str(e)),
                   file=sys.stderr)
        return 1
    return 0


def _validate_network(cfp, network_section):
    dhcpip = cfp.get_attribute(network_section, PVMConfigData.DHCPIP)
    if dhcpip == 'DISABLED':
        dhcpip = ''
    hostname = cfp.get_attribute(network_section, PVMConfigData.HOSTNAME)
    domainname = cfp.get_attribute(network_section, PVMConfigData.DOMAIN)
    ipaddr = cfp.get_attribute(network_section, PVMConfigData.IPADDR)
    netmask = cfp.get_attribute(network_section, PVMConfigData.NETMASK)
    gateway = cfp.get_attribute(network_section, PVMConfigData.GATEWAY)
    dns1 = cfp.get_attribute(network_section, PVMConfigData.DNS1)
    dns2 = cfp.get_attribute(network_section, PVMConfigData.DNS2)
    dns3 = cfp.get_attribute(network_section, PVMConfigData.DNS3)
    networkobj = Network(hostname, domainname, ipaddr, netmask,
                         gateway, dhcpip, dns1, dns2, dns3)
    try:
        Network.validate(networkobj)
    except BadNetworkSettingsException as e:
        six.print_(_("Error found in %s section: %s") % (network_section,
                                                         str(e)),
                   file=sys.stderr)
        return 1
    return 0


def _validate_phys_io(cfp, section):
    rc = 0
    val = cfp.get_attribute(section, PVMConfigData.INFO)
    if val:
        result = {}
        try:
            result = ast.literal_eval(val)
        except (SyntaxError, ValueError) as e:
            six.print_(_("Error found in %s section, field %s: %s") %
                       (section, PVMConfigData.INFO, str(e)),
                       file=sys.stderr)
            rc = 1
        if not isinstance(result, dict):
            six.print_(_("Error found in %s section, field %s:"
                         " syntax error") %
                       (section, PVMConfigData.INFO), file=sys.stderr)
            rc = 1
    return rc


def _validate_vios_link_aggregation(cfp, section):
    val = cfp.get_attribute(section, PVMConfigData.LINK_AGGREGATION_TYPE)
    if val in ['standard', 'round_robin', '8023ad']:
        return 0
    else:
        six.print_(_("Error found in %s section: "
                     "%s") % (section, PVMConfigData.LINK_AGGREGATION_TYPE),
                   file=sys.stderr)
        six.print_(_("Invalid value used : %s") % val, file=sys.stderr)
        return 1


def _validate_pvid(cfp, section):
    val = cfp.get_attribute(section, PVMConfigData.PVID)
    try:
        int(val)
    except ValueError:
        six.print_(_("Error found in %s section: "
                     "%s") % (section, PVMConfigData.PVID),
                   file=sys.stderr)
        six.print_(_("Invalid value used : %s") % val, file=sys.stderr)
        return 1
    return 0


def _validate_sdn_link_aggregation(cfp, section):
    val1 = cfp.get_attribute(section, PVMConfigData.LINK_AGGREGATION_TYPE)
    val2 = cfp.get_attribute(section,
                             PVMConfigData.FAILURE_DETECTION_ALGORITHM)
    if (val1 not in ['Server Load Balance', 'Active/Backup', 'LACP Active',
                     'LACP Passive']):
        six.print_(_("Error found in %s section: "
                     "%s") % (section, PVMConfigData.LINK_AGGREGATION_TYPE),
                   file=sys.stderr)
        six.print_(_("Invalid value used : %s") % val1, file=sys.stderr)
        return 1
    elif (val2 not in ['Miimon', 'Carrier']):
        six.print_(_("Error found in %s section: "
                     "%s") % (section,
                              PVMConfigData.FAILURE_DETECTION_ALGORITHM),
                   file=sys.stderr)
        six.print_(_("Invalid value used : %s") % val2, file=sys.stderr)
        return 1
    else:
        return 0


def _validate_proc_mem(cfp, section):
    rc = 0
    virtprocmin = cfp.get_attribute(section, PVMConfigData.VIRTUALPROCMIN)
    virtprocmax = cfp.get_attribute(section, PVMConfigData.VIRTUALPROCMAX)
    virtprocdesired = cfp.get_attribute(section,
                                        PVMConfigData.VIRTUALPROCDESIRED)
    virtprocinfo = VirtProcInfo(virtprocmin, virtprocmax, virtprocdesired)
    try:
        virtprocinfo.validate()
    except BadVirtProcSettingsException as e:
        six.print_(_("Error in section %s:\n%s") % (section, str(e)),
                   file=sys.stderr)
        rc = 1
    desiredmem = cfp.get_attribute(section, PVMConfigData.DESIREDMEMORY)
    minmem = cfp.get_attribute(section, PVMConfigData.MINMEMORY)
    maxmem = cfp.get_attribute(section, PVMConfigData.MAXMEMORY)
    lmbsize = cfp.get_attribute(PVMConfigData.SYSCONFIG_SECTION,
                                PVMConfigData.LMBSIZE)
    meminfo = MemInfo(minmem, maxmem, desiredmem, lmbsize)
    try:
        meminfo.validate()
    except BadMemorySettingsException as e:
        six.print_(_("Error in section %s:\n%s") % (section, str(e)),
                   file=sys.stderr)
        rc = 1
    procunitmin = cfp.get_attribute(section, PVMConfigData.PROCUNITMIN)
    procunitmax = cfp.get_attribute(section, PVMConfigData.PROCUNITMAX)
    procunitdesired = cfp.get_attribute(section, PVMConfigData.PROCUNITDESIRED)
    procunitinfo = ProcUnitInfo(procunitmin, procunitmax, procunitdesired)
    try:
        procunitinfo.validate()
    except BadProcUnitException as e:
        six.print_(_("Error in section %s:\n%s") % (section, str(e)),
                   file=sys.stderr)
        rc = 1
    return rc


def _validate_target_disk(cfp, section):
    selectedDisks = cfp.get_attribute(section,
                                      PVMConfigData.TARGET_DISK_INSTALL)

    try:
        TargetDisk.validate(selectedDisks)
    except StorageDevicesException as e:
        six.print_(_("Error found in %s section: %s") % (section, str(e)),
                   file=sys.stderr)
        return 1
    return 0


def validate(cfp, section, greenfield, installmode):
    rc = 0
    temprc = 0
    pcd_cls = PVMConfigData
    if section == 'ALL' or section == pcd_cls.CREDENTIALS_SECTION:
        temprc = _validate_user(cfp)
    rc = rc | temprc
    if section == 'ALL' or section == pcd_cls.NETWORK_SETTINGS_SECTION:
        temprc = _validate_network(cfp, pcd_cls.NETWORK_SETTINGS_SECTION)
    rc = rc | temprc
    val = cfp.get_attribute(pcd_cls.PVM_CONFIG_SECTION, pcd_cls.IOREDUNDANCY)
    redundantvios = ast.literal_eval(val)

    if (greenfield and (section == 'ALL' or
                        section == pcd_cls.VIOSNIMSERVER_CONFIG_SECTION)):
        temprc = _validate_vios_install(cfp)
    rc = rc | temprc

    if (greenfield and (section == 'ALL' or
                        section == pcd_cls.SDN_CONFIG_SECTION)):
        temprc = _validate_sdn_configuration(cfp)
    rc = rc | temprc

    if (greenfield and installmode != PVMConfigData.SDE and
            (section == 'ALL' or
             section == PVMConfigData.VIOS1_NETW_SETTINGS_SECTION)):
        temprc = _validate_network(cfp, pcd_cls.VIOS1_NETW_SETTINGS_SECTION)
    rc = rc | temprc

    docheck = (greenfield and (section == 'ALL' or
                               section == pcd_cls.VIOS2_NETW_SETTINGS_SECTION)
               and installmode != PVMConfigData.SDE)

    if redundantvios and docheck:
        temprc = _validate_network(cfp, pcd_cls.VIOS2_NETW_SETTINGS_SECTION)
    rc = rc | temprc

    if (greenfield and (section == 'ALL' or
                        section == pcd_cls.VIOS_CONFIG_SECTION)):
        temprc = _validate_proc_mem(cfp, pcd_cls.VIOS_CONFIG_SECTION)
    rc = rc | temprc

    docheck = (greenfield and (section == 'ALL' or section ==
                               pcd_cls.VIOS1_PHYS_IO_CONFIG_SECTION))
    if docheck:
        temprc = _validate_phys_io(cfp, pcd_cls.VIOS1_PHYS_IO_CONFIG_SECTION)
    rc = rc | temprc

    docheck = (greenfield and (section == 'ALL' or section ==
                               pcd_cls.VIOS2_PHYS_IO_CONFIG_SECTION))
    if redundantvios and docheck:
        temprc = _validate_phys_io(cfp, pcd_cls.VIOS2_PHYS_IO_CONFIG_SECTION)
    rc = rc | temprc

    docheck = (greenfield and (section == 'ALL' or
                               section == pcd_cls.VIOS1_NETWORK_BRIDGE))
    if docheck:
        temprc = _validate_vios_link_aggregation(cfp,
                                                 pcd_cls.VIOS1_NETWORK_BRIDGE)
    rc = rc | temprc
    if docheck:
        temprc = _validate_pvid(cfp, pcd_cls.VIOS1_NETWORK_BRIDGE)
    rc = rc | temprc

    docheck = (greenfield and (section == 'ALL' or
                               section == pcd_cls.VIOS2_NETWORK_BRIDGE))
    if redundantvios and docheck:
        temprc = _validate_vios_link_aggregation(cfp,
                                                 pcd_cls.VIOS2_NETWORK_BRIDGE)
    rc = rc | temprc

    # configuration variables for SDN related installation steps
    sdn_section = pcd_cls.SDN_CONFIG_SECTION

    sdn_enabled = \
        cfp.get_attribute_notstr(sdn_section, pcd_cls.SDN_ENABLED)
    # sdn_redun_enabled = \
    #     cfgparser.get_attribute_notstr(sdn_section,
    #                                    pcd_cls.SDN_REDUN_ENABLED)

    docheck = (greenfield and sdn_enabled and
               (section == 'ALL' or
                section == pcd_cls.SDN1_NETWORK_BRIDGE))
    if docheck:
        temprc = _validate_sdn_link_aggregation(cfp,
                                                pcd_cls.SDN1_NETWORK_BRIDGE)

    if section == 'ALL' or section == pcd_cls.PVM_CONFIG_SECTION:
        temprc = _validate_proc_mem(cfp, pcd_cls.PVM_CONFIG_SECTION)
    rc = rc | temprc

    if section == 'ALL':
        temprc = check_duplicate_entries(cfp, greenfield, redundantvios,
                                         installmode)
    rc = rc | temprc

    if installmode == PVMConfigData.SDE and (section == 'ALL' or
                                             section ==
                                             pcd_cls.PVM_CONFIG_SECTION):
        temprc = _validate_target_disk(cfp, pcd_cls.PVM_CONFIG_SECTION)
    rc = rc | temprc

    return rc


if __name__ == '__main__':
    cfgfile = ""
    option = ""
    attr = ""
    val = ""
    sect = "ALL"
    rawmode = False
    greenfield = False
    installmode = ""
    gettext.textdomain('pvm-install')
    locale.setlocale(locale.LC_ALL, None)

    try:
        options, remainder = getopt.getopt(sys.argv[1:], 'f:o:a:v:s:m:rgh',
                                           ['help'])
    except getopt.GetoptError as e:
        print(str(e))
        printusage()

    for opt, arg in options:
        if opt in ('-h', '--help'):
            printusage()
        elif opt in ('-f'):
            cfgfile = arg
        elif opt in ('-r'):
            rawmode = True
        elif opt in ('-s'):
            sect = arg
        elif opt in ('-o'):
            option = arg
        elif opt in ('-a'):
            attr = arg
        elif opt in ('-v'):
            val = arg
        elif opt in ('-g'):
            greenfield = True
        elif opt in ('-m'):
            installmode = arg

    if option != 'm' and val:
        six.print_(_("val arguments are only valid when "
                     "-o m is used."), file=sys.stderr)
        sys.exit(1)
    elif option not in ['m', 'd'] and attr:
        six.print_(_("attr arguments are only valid when "
                     "-o m or -o d is used."), file=sys.stderr)
        sys.exit(1)
    try:
        if rawmode:
            pcd_obj = PVMConfigData(cfgfile, raw=True)
        else:
            pcd_obj = PVMConfigData(cfgfile)
    except:
        six.print_(_("Error parsing %s\n") % cfgfile, file=sys.stderr)
        raise
        sys.exit(1)

    rc = 0
    if option == 'd':
        if attr:
            dumpattr(pcd_obj, attr)
        else:
            dumpcfg(pcd_obj, sect)
    elif option == 'm':
        modify(pcd_obj, attr, val)
    elif option == 'v':
        rc = validate(pcd_obj, sect, greenfield, installmode)
    sys.exit(rc)
