# IBM CONFIDENTIAL
#
# OBJECT CODE ONLY SOURCE MATERIALS
#
# COPYRIGHT International Business Machines Corp. 2015
# All Rights Reserved
#
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.

from snack import Grid
from snack import GridForm
from snack import SingleRadioButton
from snack import TextboxReflowed
from gettext import gettext as _
from wizpkg.wizardpanel import WizardPanel
from wizpkg.extendedwizardpanels import buildButtonBar


class WelcomeWizardPanel(WizardPanel):
    '''
    This class extends the base WizardPanel and uses
    one or more RadioButton as widget in its gridform.
             +-------------------------------+
             |     some title                |
             +-------------------------------+
             | Here are some instructions..  |
             |                               |
             | [ ] Some text                 |
             | [ ] Some other text           |
             | ...                           |
             |                               |
             | <back>   <next>   <cancel>    |
             +-------------------------------+
    '''
    INSTALL = 'INSTALL'
    RECOVER = 'RECOVER'

    def __init__(self, screen, name, paneltitle, msg,
                 show_vios, position, panelaction):
        '''
        Init method
        :param screen: SnackScreen instance
        :param name: Name of the panel
        :param paneltitle: Panel title displayed to user
        :param msg: Message displayed to user as instructions
        :param show_vios: Flag to present proper message for installmode
        :param position: Panel position in wizard
        :param panelaction: Function to called when user transition out
        '''
        gf = GridForm(screen, paneltitle, 1, 3)
        tb = TextboxReflowed(WizardPanel.WIDTH, msg)

        install_button_msg = _("Choose to perform an installation.")
        recover_button_msg = _("Choose to repair a system.")
        if show_vios:
            installmsg = _("This will perform an installation of the NovaLink "
                           "partition, its core components and REST APIs, and "
                           "all needed Virtual I/O servers.")
            recovermsg = _("This will repair the system by performing a "
                           "rescue/repair of existing Virtual I/O servers and "
                           "NovaLink partitions.\nChoose this option if "
                           "PowerVM is already installed but is corrupted or "
                           "there is a failure.")

        else:
            installmsg = _("This will perform an installation of the NovaLink "
                           "partition, its core components and REST APIs")
            recovermsg = _("This will repair the system by performing a "
                           "rescue/repair of existing NovaLink partitions.\n"
                           "Choose this option if PowerVM is already "
                           "installed but is corrupted or there is a failure.")

        self.installrb = SingleRadioButton(install_button_msg, 0, 1)
        self.recoverrb = SingleRadioButton(recover_button_msg,
                                           self.installrb, 0)
        g = Grid(1, 5)
        g.setField(self.installrb, 0, 0, padding=(0, 0, 1, 0), anchorLeft=1)
        g.setField(TextboxReflowed(WizardPanel.WIDTH, installmsg), 0, 1,
                   padding=(4, 0, 1, 1), anchorLeft=1)
        g.setField(self.recoverrb, 0, 2, padding=(0, 0, 1, 0), anchorLeft=1)
        g.setField(TextboxReflowed(WizardPanel.WIDTH, recovermsg), 0, 3,
                   padding=(4, 0, 1, 1), anchorLeft=1)
        gf.add(tb, 0, 0, padding=(0, 0, 1, 1), anchorLeft=1)
        gf.add(g, 0, 1, growy=1, anchorTop=0)
        bb = buildButtonBar(screen, position)
        gf.add(bb, 0, 2, padding=(0, 1, 0, 0), growx=1)
        self.buttonBar = bb
        super(WelcomeWizardPanel, self).__init__(screen=screen,
                                                 name=name, gridform=gf,
                                                 panelaction=panelaction)

    def getValues(self):
        '''
        Obtain values selected
        '''
        if self.installrb.selected():
            return WelcomeWizardPanel.INSTALL
        elif self.recoverrb.selected():
            return WelcomeWizardPanel.RECOVER


class DefaultOrCustomWizardPanel(WizardPanel):

    DEFAULT = 'DEFAULT'
    CUSTOM = 'CUSTOM'

    def __init__(self, screen, name, paneltitle, msg, position, panelaction):
        '''
        Init method
        :param screen: SnackScreen instance
        :param name: Name of the panel
        :param paneltitle: Panel title displayed to user
        :param msg: Message displayed to user as instructions
        :param position: Panel position in wizard
        :param panelaction: Function to called when user transition out
        '''
        gf = GridForm(screen, paneltitle, 1, 3)
        tb = TextboxReflowed(WizardPanel.WIDTH, msg)

        default_button_msg = _("Choose to accept all available defaults.")
        custom_button_msg = _("Choose to provide custom values.")
        defaultmsg = _("This option will cause some panels to be skipped.")
        custommsg = _("This option will cause all panels to be displayed.")
        self.defaultrb = SingleRadioButton(default_button_msg, 0, 1)
        self.customrb = SingleRadioButton(custom_button_msg, self.defaultrb, 0)
        g = Grid(1, 4)
        g.setField(self.defaultrb, 0, 0, padding=(0, 0, 1, 0), anchorLeft=1)
        g.setField(TextboxReflowed(WizardPanel.WIDTH, defaultmsg), 0, 1,
                   padding=(4, 0, 1, 1), anchorLeft=1)
        g.setField(self.customrb, 0, 2, padding=(0, 0, 1, 0), anchorLeft=1)
        g.setField(TextboxReflowed(WizardPanel.WIDTH, custommsg), 0, 3,
                   padding=(4, 0, 1, 1), anchorLeft=1)
        gf.add(tb, 0, 0, padding=(0, 0, 0, 1), anchorTop=0)
        gf.add(g, 0, 1, growy=1, anchorTop=0)
        bb = buildButtonBar(screen, position)
        gf.add(bb, 0, 2, padding=(0, 1, 0, 0), growx=1)
        self.buttonBar = bb
        super(DefaultOrCustomWizardPanel,
              self).__init__(screen=screen, name=name, gridform=gf,
                             panelaction=panelaction)

    def getValues(self):
        '''
        Obtain values selected
        '''
        if self.defaultrb.selected():
            return DefaultOrCustomWizardPanel.DEFAULT
        elif self.customrb.selected():
            return DefaultOrCustomWizardPanel.CUSTOM
