# IBM CONFIDENTIAL
#
# OBJECT CODE ONLY SOURCE MATERIALS
#
# COPYRIGHT International Business Machines Corp. 2015
# All Rights Reserved
#
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.

# This file contains class that implements a command execution with
# a progress bar.

from gettext import gettext as _
from snack import GridForm
from snack import Scale
from snack import Textbox
from snack import TextboxReflowed
from wizpkg.wizardpanel import WizardPanel
import os
import sys


class CommandProgressWindow(object):
    '''
    This class provides a progress bar and a text area to display
    status of command being executed. If update is set to True
    a progress bar will be included
    The progress bar uses the snack Scale widget
    '''

    def __init__(self, screen, title, text, total,
                 step, update=False):
        '''
        Instantiate a Window with progress bar
        :param screen: SnackScreen instance
        :param title: Title to display to user
        :param text: Displayed text, for example what is executing
        :param total: The total number of steps involved
        :param step: The incremental each time the bar is updated
        :param update: Flag to indicate whether a progress bar is included
        '''
        self.screen = screen
        self.count = 0
        self.multiplier = 1
        self.step = step
        # May need to pass this in, currently set to 60
        width = 60
        self.total = total
        # Text box area to show command
        # Only 1 line will be shown.
        self.info = Textbox(width, 1, "", wrap=1)

        if (len(text) > width):
            width = len(text)

        t = TextboxReflowed(width, text)

        self.grid = GridForm(self.screen, title, 1, 6)
        if self.total == 1.0:
            self.multiplier = 100

        # bar -> scale(width, full_value)
        self.scale = Scale(int(width),
                           int(float(self.total) * self.multiplier))
        self.grid.add(t, 0, 1, padding=(0, 0, 0, 1), anchorLeft=1)
        if update:
            self.grid.add(self.scale, 0, 2)

        self.grid.add(self.info, 0, 3, (0, 1, 0, 0))
        self.grid.draw()
        self.screen.refresh()

    def pop(self):
        self.screen.popWindow()
        self.screen.refresh()
        del self.scale
        self.scale = None

    def update(self):
        if abs(self.count - self.total) <= 0.001:
            self.count = 0
        else:
            # 10 % increment each time
            self.count = self.count + self.step
        self.set(self.count)

    def set(self, amount):
        self.scale.set(int(float(amount) * self.multiplier))
        self.screen.refresh()

    def refresh(self):
        pass

    def setText(self, text):
        self.info.setText(text)
        self.screen.refresh()


class ExecWithProgress(object):

    @staticmethod
    def runcmd(command, argv, screen, stdin=0, stdout=1, stderr=2, pBar=None):
        argv = list(argv)
        if isinstance(stdin, str):
            if os.access(stdin, os.R_OK):
                stdin = open(stdin)
            else:
                stdin = sys.stdin

        if isinstance(stdout, str):
            stdout = open(stdout, "w+")
        else:
            stdout = sys.stdout
        if isinstance(stderr, str):
            stderr = open(stderr, "w+")
        else:
            stderr = sys.stderr

        # Create a pipe to facilitate capturing output of command

        p = os.pipe()
        childpid = os.fork()
        if not childpid:
            os.close(p[0])
            os.dup2(p[1], 1)
            os.dup2(stderr.fileno(), 2)
            os.dup2(stdin, 0)
            os.close(stdin)
            os.close(p[1])
            stderr.close()

            os.execvp(command, [command] + argv)
            os._exit(1)

        os.close(p[1])
        while 1:
            try:
                s = os.read(p[0], 50)
            except OSError as e:
                num = e.args[0]
                if num != 4:
                    raise IOError(str(e))

            # Update window with text and progress bar value
            if pBar:
                pBar.setText(str(s, encoding="utf-8"))
                pBar.update()

            if len(s) < 1:
                break
        try:
            status = os.waitpid(childpid, 0)[1]
        except OSError as e:
            WizardPanel.showError(screen, _("An exception has occurred from "
                                            "running %s" % command))
        pBar and pBar.pop()

        if status is None:
            return 0
        # One should never get here
        if os.WIFEXITED(status):
            return os.WEXITSTATUS(status)
        return 1
